@@ -64,25 +64,12 @@
 #include <drm/drm_ioctl.h>
 #endif
 
-#if defined(NV_LINUX_APERTURE_H_PRESENT)
-#include <linux/aperture.h>
-#endif
-
-#if defined(NV_DRM_DRM_APERTURE_H_PRESENT)
+#if defined(NV_DRM_FBDEV_GENERIC_AVAILABLE)
 #include <drm/drm_aperture.h>
-#endif
-
-#if defined(NV_DRM_FBDEV_AVAILABLE)
 #include <drm/drm_fb_helper.h>
 #endif
 
-#if defined(NV_DRM_DRM_CLIENT_SETUP_H_PRESENT)
-#include <drm/drm_client_setup.h>
-#endif
-
-#if defined(NV_DRM_DRM_FBDEV_TTM_H_PRESENT)
-#include <drm/drm_fbdev_ttm.h>
-#elif defined(NV_DRM_DRM_FBDEV_GENERIC_H_PRESENT)
+#if defined(NV_DRM_DRM_FBDEV_GENERIC_H_PRESENT)
 #include <drm/drm_fbdev_generic.h>
 #endif
 
@@ -137,7 +124,6 @@ static const char* nv_get_input_colorspace_name(
 
 #if defined(NV_DRM_ATOMIC_MODESET_AVAILABLE)
 
-#if defined(NV_DRM_OUTPUT_POLL_CHANGED_PRESENT)
 static void nv_drm_output_poll_changed(struct drm_device *dev)
 {
     struct drm_connector *connector = NULL;
@@ -181,7 +167,6 @@ static void nv_drm_output_poll_changed(struct drm_device *dev)
     nv_drm_connector_list_iter_end(&conn_iter);
 #endif
 }
-#endif /* NV_DRM_OUTPUT_POLL_CHANGED_PRESENT */
 
 static struct drm_framebuffer *nv_drm_framebuffer_create(
     struct drm_device *dev,
@@ -219,9 +204,7 @@ static const struct drm_mode_config_funcs nv_mode_config_funcs = {
     .atomic_check  = nv_drm_atomic_check,
     .atomic_commit = nv_drm_atomic_commit,
 
-    #if defined(NV_DRM_OUTPUT_POLL_CHANGED_PRESENT)
     .output_poll_changed = nv_drm_output_poll_changed,
-    #endif
 };
 
 static void nv_drm_event_callback(const struct NvKmsKapiEvent *event)
@@ -390,19 +373,15 @@ static int nv_drm_create_properties(struct nv_drm_device *nv_dev)
         len++;
     }
 
-#if defined(NV_LINUX_NVHOST_H_PRESENT) && defined(CONFIG_TEGRA_GRHOST)
-    if (!nv_dev->supportsSyncpts) {
-        return 0;
+    if (nv_dev->supportsSyncpts) {
+        nv_dev->nv_out_fence_property =
+            drm_property_create_range(nv_dev->dev, DRM_MODE_PROP_ATOMIC,
+                    "NV_DRM_OUT_FENCE_PTR", 0, U64_MAX);
+        if (nv_dev->nv_out_fence_property == NULL) {
+            return -ENOMEM;
+        }
     }
 
-    nv_dev->nv_out_fence_property =
-        drm_property_create_range(nv_dev->dev, DRM_MODE_PROP_ATOMIC,
-            "NV_DRM_OUT_FENCE_PTR", 0, U64_MAX);
-    if (nv_dev->nv_out_fence_property == NULL) {
-        return -ENOMEM;
-    }
-#endif
-
     nv_dev->nv_input_colorspace_property =
         drm_property_create_enum(nv_dev->dev, 0, "NV_INPUT_COLORSPACE",
                                  enum_list, len);
@@ -497,7 +476,7 @@ static int nv_drm_load(struct drm_device *dev, unsigned long flags)
         return -ENODEV;
     }
 
-#if defined(NV_DRM_FBDEV_AVAILABLE)
+#if defined(NV_DRM_FBDEV_GENERIC_AVAILABLE)
     /*
      * If fbdev is enabled, take modeset ownership now before other DRM clients
      * can take master (and thus NVKMS ownership).
@@ -625,7 +604,7 @@ static void __nv_drm_unload(struct drm_device *dev)
 
     /* Release modeset ownership if fbdev is enabled */
 
-#if defined(NV_DRM_FBDEV_AVAILABLE)
+#if defined(NV_DRM_FBDEV_GENERIC_AVAILABLE)
     if (nv_dev->hasFramebufferConsole) {
         drm_atomic_helper_shutdown(dev);
         nvKms->releaseOwnership(nv_dev->pDevice);
@@ -823,6 +802,14 @@ static int nv_drm_get_dev_info_ioctl(struct drm_device *dev,
     return 0;
 }
 
+static int nv_drm_get_drm_file_unique_id_ioctl(struct drm_device *dev,
+                                               void *data, struct drm_file *filep)
+{
+    struct drm_nvidia_get_drm_file_unique_id_params *params = data;
+    params->id = (u64)(filep->driver_priv);
+    return 0;
+}
+
 static int nv_drm_dmabuf_supported_ioctl(struct drm_device *dev,
                                          void *data, struct drm_file *filep)
 {
@@ -876,18 +863,13 @@ static int nv_drm_get_dpy_id_for_connector_id_ioctl(struct drm_device *dev,
                                                     struct drm_file *filep)
 {
     struct drm_nvidia_get_dpy_id_for_connector_id_params *params = data;
-    struct drm_connector *connector;
-    struct nv_drm_connector *nv_connector;
-    int ret = 0;
-
-    if (!drm_core_check_feature(dev, DRIVER_MODESET)) {
-        return -EOPNOTSUPP;
-    }
-
     // Importantly, drm_connector_lookup (with filep) will only return the
     // connector if we are master, a lessee with the connector, or not master at
     // all. It will return NULL if we are a lessee with other connectors.
-    connector = nv_drm_connector_lookup(dev, filep, params->connectorId);
+    struct drm_connector *connector =
+        nv_drm_connector_lookup(dev, filep, params->connectorId);
+    struct nv_drm_connector *nv_connector;
+    int ret = 0;
 
     if (!connector) {
         return -EINVAL;
@@ -920,11 +902,6 @@ static int nv_drm_get_connector_id_for_dpy_id_ioctl(struct drm_device *dev,
     int ret = -EINVAL;
 #if defined(NV_DRM_CONNECTOR_LIST_ITER_PRESENT)
     struct drm_connector_list_iter conn_iter;
-#endif
-    if (!drm_core_check_feature(dev, DRIVER_MODESET)) {
-        return -EOPNOTSUPP;
-    }
-#if defined(NV_DRM_CONNECTOR_LIST_ITER_PRESENT)
     nv_drm_connector_list_iter_begin(dev, &conn_iter);
 #endif
 
@@ -1137,10 +1114,6 @@ static int nv_drm_grant_permission_ioctl(struct drm_device *dev, void *data,
 {
     struct drm_nvidia_grant_permissions_params *params = data;
 
-    if (!drm_core_check_feature(dev, DRIVER_MODESET)) {
-        return -EOPNOTSUPP;
-    }
-
     if (params->type == NV_DRM_PERMISSIONS_TYPE_MODESET) {
         return nv_drm_grant_modeset_permission(dev, params, filep);
     } else if (params->type == NV_DRM_PERMISSIONS_TYPE_SUB_OWNER) {
@@ -1306,10 +1279,6 @@ static int nv_drm_revoke_permission_ioctl(struct drm_device *dev, void *data,
 {
     struct drm_nvidia_revoke_permissions_params *params = data;
 
-    if (!drm_core_check_feature(dev, DRIVER_MODESET)) {
-        return -EOPNOTSUPP;
-    }
-
     if (params->type == NV_DRM_PERMISSIONS_TYPE_MODESET) {
         if (!params->dpyId) {
             return -EINVAL;
@@ -1339,6 +1308,17 @@ static void nv_drm_postclose(struct drm_device *dev, struct drm_file *filep)
 }
 #endif /* NV_DRM_ATOMIC_MODESET_AVAILABLE */
 
+static int nv_drm_open(struct drm_device *dev, struct drm_file *filep)
+{
+    _Static_assert(sizeof(filep->driver_priv) >= sizeof(u64),
+                   "filep->driver_priv can not hold an u64");
+    static atomic64_t id = ATOMIC_INIT(0);
+
+    filep->driver_priv = (void *)atomic64_inc_return(&id);
+
+    return 0;
+}
+
 #if defined(NV_DRM_MASTER_HAS_LEASES)
 static struct drm_master *nv_drm_find_lessee(struct drm_master *master,
                                              int lessee_id)
@@ -1564,10 +1544,6 @@ static const struct file_operations nv_drm_fops = {
     .read           = drm_read,
 
     .llseek         = noop_llseek,
-
-#if defined(NV_FILE_OPERATIONS_FOP_UNSIGNED_OFFSET_PRESENT)
-    .fop_flags   = FOP_UNSIGNED_OFFSET,
-#endif
 };
 
 static const struct drm_ioctl_desc nv_drm_ioctls[] = {
@@ -1586,6 +1562,9 @@ static const struct drm_ioctl_desc nv_drm_ioctls[] = {
     DRM_IOCTL_DEF_DRV(NVIDIA_GET_DEV_INFO,
                       nv_drm_get_dev_info_ioctl,
                       DRM_RENDER_ALLOW|DRM_UNLOCKED),
+    DRM_IOCTL_DEF_DRV(NVIDIA_GET_DRM_FILE_UNIQUE_ID,
+                      nv_drm_get_drm_file_unique_id_ioctl,
+                      DRM_RENDER_ALLOW|DRM_UNLOCKED),
 
 #if defined(NV_DRM_FENCE_AVAILABLE)
     DRM_IOCTL_DEF_DRV(NVIDIA_FENCE_SUPPORTED,
@@ -1668,6 +1647,9 @@ static struct drm_driver nv_drm_driver = {
     .driver_features        =
 #if defined(NV_DRM_DRIVER_PRIME_FLAG_PRESENT)
                                DRIVER_PRIME |
+#endif
+#if defined(NV_DRM_SYNCOBJ_FEATURES_PRESENT)
+                               DRIVER_SYNCOBJ | DRIVER_SYNCOBJ_TIMELINE |
 #endif
                                DRIVER_GEM  | DRIVER_RENDER,
 
@@ -1679,14 +1661,14 @@ static struct drm_driver nv_drm_driver = {
     .num_ioctls             = ARRAY_SIZE(nv_drm_ioctls),
 
 /*
- * linux-next commit 71a7974ac701 ("drm/prime: Unexport helpers for fd/handle
- * conversion") unexports drm_gem_prime_handle_to_fd() and
+ * Linux kernel v6.6 commit 71a7974ac701 ("drm/prime: Unexport helpers
+ * for fd/handle conversion") unexports drm_gem_prime_handle_to_fd() and
  * drm_gem_prime_fd_to_handle().
  *
- * Prior linux-next commit 6b85aa68d9d5 ("drm: Enable PRIME import/export for
- * all drivers") made these helpers the default when .prime_handle_to_fd /
- * .prime_fd_to_handle are unspecified, so it's fine to just skip specifying
- * them if the helpers aren't present.
+ * Prior Linux kernel v6.6 commit 6b85aa68d9d5 ("drm: Enable PRIME
+ * import/export for all drivers") made these helpers the default when
+ * .prime_handle_to_fd / .prime_fd_to_handle are unspecified, so it's fine
+ * to just skip specifying them if the helpers aren't present.
  */
 #if NV_IS_EXPORT_SYMBOL_PRESENT_drm_gem_prime_handle_to_fd
     .prime_handle_to_fd     = drm_gem_prime_handle_to_fd,
@@ -1720,6 +1702,7 @@ static struct drm_driver nv_drm_driver = {
 #if defined(NV_DRM_ATOMIC_MODESET_AVAILABLE)
     .postclose              = nv_drm_postclose,
 #endif
+    .open                   = nv_drm_open,
 
     .fops                   = &nv_drm_fops,
 
@@ -1730,20 +1713,13 @@ static struct drm_driver nv_drm_driver = {
     .name                   = "nvidia-drm",
 
     .desc                   = "NVIDIA DRM driver",
-
-#if defined(NV_DRM_DRIVER_HAS_DATE)
     .date                   = "20160202",
-#endif
 
 #if defined(NV_DRM_DRIVER_HAS_DEVICE_LIST)
     .device_list            = LIST_HEAD_INIT(nv_drm_driver.device_list),
 #elif defined(NV_DRM_DRIVER_HAS_LEGACY_DEV_LIST)
     .legacy_dev_list        = LIST_HEAD_INIT(nv_drm_driver.legacy_dev_list),
 #endif
-// XXX implement nvidia-drm's own .fbdev_probe callback that uses NVKMS kapi directly
-#if defined(NV_DRM_FBDEV_AVAILABLE) && defined(DRM_FBDEV_TTM_DRIVER_OPS)
-    DRM_FBDEV_TTM_DRIVER_OPS,
-#endif
 };
 
 
@@ -1785,6 +1761,7 @@ void nv_drm_register_drm_device(const nv_gpu_info_t *gpu_info)
     struct nv_drm_device *nv_dev = NULL;
     struct drm_device *dev = NULL;
     struct device *device = gpu_info->os_device_ptr;
+    bool bus_is_pci;
 
     DRM_DEBUG(
         "Registering device for NVIDIA GPU ID 0x08%x",
@@ -1818,7 +1795,7 @@ void nv_drm_register_drm_device(const nv_gpu_info_t *gpu_info)
     dev->dev_private = nv_dev;
     nv_dev->dev = dev;
 
-    bool bus_is_pci =
+    bus_is_pci =
 #if defined(NV_LINUX)
         device->bus == &pci_bus_type;
 #elif defined(NV_BSD)
@@ -1838,34 +1815,22 @@ void nv_drm_register_drm_device(const nv_gpu_info_t *gpu_info)
         goto failed_drm_register;
     }
 
-#if defined(NV_DRM_FBDEV_AVAILABLE)
+#if defined(NV_DRM_FBDEV_GENERIC_AVAILABLE)
     if (nv_drm_fbdev_module_param &&
         drm_core_check_feature(dev, DRIVER_MODESET)) {
 
         if (bus_is_pci) {
             struct pci_dev *pdev = to_pci_dev(device);
 
-#if defined(NV_DRM_APERTURE_REMOVE_CONFLICTING_PCI_FRAMEBUFFERS_PRESENT)
-
 #if defined(NV_DRM_APERTURE_REMOVE_CONFLICTING_PCI_FRAMEBUFFERS_HAS_DRIVER_ARG)
             drm_aperture_remove_conflicting_pci_framebuffers(pdev, &nv_drm_driver);
 #else
             drm_aperture_remove_conflicting_pci_framebuffers(pdev, nv_drm_driver.name);
 #endif
-
-#elif defined(NV_APERTURE_REMOVE_CONFLICTING_PCI_DEVICES_PRESENT)
-            aperture_remove_conflicting_pci_devices(pdev, nv_drm_driver.name);
-#endif
         }
-        #if defined(NV_DRM_CLIENT_AVAILABLE)
-	    drm_client_setup(dev, NULL);
-        #elif defined(NV_DRM_FBDEV_TTM_AVAILABLE)
-        drm_fbdev_ttm_setup(dev, 32);
-        #elif defined(NV_DRM_FBDEV_GENERIC_AVAILABLE)
         drm_fbdev_generic_setup(dev, 32);
-        #endif
     }
-#endif /* defined(NV_DRM_FBDEV_AVAILABLE) */
+#endif /* defined(NV_DRM_FBDEV_GENERIC_AVAILABLE) */
 
     /* Add NVIDIA-DRM device into list */
 
@@ -2007,12 +1972,12 @@ void nv_drm_suspend_resume(NvBool suspend)
 
         if (suspend) {
             drm_kms_helper_poll_disable(dev);
-#if defined(NV_DRM_FBDEV_AVAILABLE)
+#if defined(NV_DRM_FBDEV_GENERIC_AVAILABLE)
             drm_fb_helper_set_suspend_unlocked(dev->fb_helper, 1);
 #endif
             drm_mode_config_reset(dev);
         } else {
-#if defined(NV_DRM_FBDEV_AVAILABLE)
+#if defined(NV_DRM_FBDEV_GENERIC_AVAILABLE)
             drm_fb_helper_set_suspend_unlocked(dev->fb_helper, 0);
 #endif
             drm_kms_helper_poll_enable(dev);
