@@ -130,12 +130,27 @@ static NV_STATUS block_migrate_map_unmapped_pages(uvm_va_block_t *va_block,
     uvm_tracker_t local_tracker = UVM_TRACKER_INIT();
     NV_STATUS status = NV_OK;
     NV_STATUS tracker_status;
-    uvm_prot_t prot = UVM_PROT_READ_WRITE_ATOMIC;
 
     // Get the mask of unmapped pages because it will change after the
     // first map operation
     uvm_va_block_unmapped_pages_get(va_block, region, &va_block_context->caller_page_mask);
 
+    if (uvm_va_block_is_hmm(va_block) && !UVM_ID_IS_CPU(dest_id)) {
+        // Do not map pages that are already resident on the CPU. This is in
+        // order to avoid breaking system-wide atomic operations on HMM. HMM's
+        // implementation of system-side atomic operations involves restricting
+        // mappings to one processor (CPU or a GPU) at a time. If we were to
+        // grant a GPU a mapping to system memory, this gets into trouble
+        // because, on the CPU side, Linux can silently upgrade PTE permissions
+        // (move from read-only, to read-write, without any MMU notifiers
+        // firing), thus breaking the model by allowing simultaneous read-write
+        // access from two separate processors. To avoid that, just don't map
+        // such pages at all, when migrating.
+        uvm_page_mask_andnot(&va_block_context->caller_page_mask,
+                             &va_block_context->caller_page_mask,
+                             uvm_va_block_resident_mask_get(va_block, UVM_ID_CPU, NUMA_NO_NODE));
+    }
+
     // Only map those pages that are not mapped anywhere else (likely due
     // to a first touch or a migration). We pass
     // UvmEventMapRemoteCauseInvalid since the destination processor of a
@@ -151,31 +166,6 @@ static NV_STATUS block_migrate_map_unmapped_pages(uvm_va_block_t *va_block,
     if (status != NV_OK)
         goto out;
 
-    if (uvm_va_block_is_hmm(va_block) && UVM_ID_IS_CPU(dest_id)) {
-        uvm_processor_id_t id;
-
-        // Do not atomically map pages that are resident on the CPU. This is in
-        // order to avoid breaking system-wide atomic operations on HMM. HMM's
-        // implementation of system-side atomic operations involves restricting
-        // mappings to one processor (CPU or a GPU) at a time. If we were to
-        // grant a GPU a mapping to system memory, this gets into trouble
-        // because, on the CPU side, Linux can silently upgrade PTE permissions
-        // (move from read-only, to read-write, without any MMU notifiers
-        // firing), thus breaking the model by allowing simultaneous read-write
-        // access from two separate processors. To avoid that, don't remote map
-        // such pages atomically, after migrating.
-        // Also note that HMM sets CPU mapping for resident pages so the mask
-        // of pages to be mapped needs to be recomputed without including the
-        // CPU mapping.
-        prot = UVM_PROT_READ_WRITE;
-        uvm_page_mask_region_fill(&va_block_context->caller_page_mask, region);
-        for_each_gpu_id_in_mask(id, &va_block->mapped) {
-            uvm_page_mask_andnot(&va_block_context->caller_page_mask,
-                                 &va_block_context->caller_page_mask,
-                                 uvm_va_block_map_mask_get(va_block, id));
-        }
-    }
-
     // Add mappings for AccessedBy processors
     //
     // No mappings within this call will operate on dest_id, so we don't
@@ -186,7 +176,7 @@ static NV_STATUS block_migrate_map_unmapped_pages(uvm_va_block_t *va_block,
                                                        dest_id,
                                                        region,
                                                        &va_block_context->caller_page_mask,
-                                                       prot,
+                                                       UVM_PROT_READ_WRITE_ATOMIC,
                                                        NULL);
 
 out:
@@ -599,7 +589,7 @@ static NV_STATUS uvm_migrate_ranges(uvm_va_space_t *va_space,
                     skipped_migrate = true;
             }
             else if (uvm_processor_mask_test(&va_range->uvm_lite_gpus, dest_id) &&
-                     !uvm_va_policy_preferred_location_equal(policy, dest_id, NUMA_NO_NODE)) {
+                     !uvm_id_equal(dest_id, policy->preferred_location)) {
                 // Don't migrate to a non-faultable GPU that is in UVM-Lite mode,
                 // unless it's the preferred location
                 status = NV_ERR_INVALID_DEVICE;
