@@ -52,8 +52,6 @@ NvU32 os_page_size  = PAGE_SIZE;
 NvU64 os_page_mask  = NV_PAGE_MASK;
 NvU8  os_page_shift = PAGE_SHIFT;
 NvBool os_cc_enabled = 0;
-NvBool os_cc_sev_snp_enabled = 0;
-NvBool os_cc_snp_vtom_enabled = 0;
 NvBool os_cc_tdx_enabled = 0;
 
 #if defined(CONFIG_DMA_SHARED_BUFFER)
@@ -402,7 +400,7 @@ NvS32 NV_API_CALL os_string_compare(const char *str1, const char *str2)
     return strcmp(str1, str2);
 }
 
-static void *os_mem_copy_custom(
+void *os_mem_copy_custom(
     void       *dstPtr,
     const void *srcPtr,
     NvU32       length
