@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -44,7 +44,7 @@ namespace DisplayPort
     #define HDCP_BCAPS_DDC_EN_BIT 0x80
     #define HDCP_BCAPS_DP_EN_BIT  0x01
     #define HDCP_I2C_CLIENT_ADDR  0x74
-    #define DSC_CAPS_SIZE         16
+    #define DEVICE_OUI_SIZE       3
 
     struct GroupImpl;
     struct ConnectorImpl;
@@ -171,7 +171,6 @@ namespace DisplayPort
 
         // Panel replay Caps
         PanelReplayCaps prCaps;
-
         bool bIsFakedMuxDevice;
         bool bIsPreviouslyFakedMuxDevice;
         bool bisMarkedForDeletion;
@@ -200,8 +199,11 @@ namespace DisplayPort
         TriState bAsyncSDPCapable;
         bool bMSAOverMSTCapable;
         bool bDscPassThroughColorFormatWar;
+        bool bSkipFakeDeviceDpcdAccess;
+
+        DeviceImpl(DPCDHAL * hal, ConnectorImpl * connector, DeviceImpl * parent, bool bSkipFakeDeviceDpcdAccess);
+        NvU64 maxModeBwRequired;
 
-        DeviceImpl(DPCDHAL * hal, ConnectorImpl * connector, DeviceImpl * parent);
         ~DeviceImpl();
 
         virtual bool isCableOk();
@@ -256,13 +258,13 @@ namespace DisplayPort
 
         virtual bool isLoop()
         {
-            DP_LOG(("isLoop implementation is pending (bug 791059)"));
+           // implementation is pending (bug 791059)
             return false;
         }
 
         virtual bool isRedundant()
         {
-            DP_LOG(("isRedundant implementation is pending (bug 791059)"));
+            // implementation is pending (bug 791059)
             return false;
         }
 
@@ -380,6 +382,11 @@ namespace DisplayPort
             return dpcdRevisionMinor >= minor;
         }
 
+        NvU64 getMaxModeBwRequired()
+        {
+            return maxModeBwRequired;
+        }
+
         virtual void queryGUID2();
 
         virtual bool getSDPExtnForColorimetrySupported();
@@ -421,7 +428,6 @@ namespace DisplayPort
         virtual void    markDeviceForDeletion() {bisMarkedForDeletion = true;};
         virtual bool    isMarkedForDeletion() {return bisMarkedForDeletion;};
         virtual bool    getRawDscCaps(NvU8 *buffer, NvU32 bufferSize);
-        virtual bool    setRawDscCaps(NvU8 *buffer, NvU32 bufferSize);
 
         virtual AuxBus::status dscCrcControl(NvBool bEnable, gpuDscCrc *dataGpu, sinkDscCrc *dataSink);
 
@@ -446,11 +452,14 @@ namespace DisplayPort
         bool isPanelReplaySupported(void);
         void getPanelReplayCaps(void);
         bool setPanelReplayConfig(panelReplayConfig prcfg);
+        bool getPanelReplayConfig(panelReplayConfig *pPrcfg);
         bool getPanelReplayStatus(PanelReplayStatus *pPrStatus);
 
         NvBool getDSCSupport();
         bool getFECSupport();
         NvBool isDSCPassThroughSupported();
+        NvBool isDynamicPPSSupported();
+        NvBool isDynamicDscToggleSupported();
         NvBool isDSCSupported();
         NvBool isDSCDecompressionSupported();
         NvBool isDSCPossible();
@@ -480,6 +489,11 @@ namespace DisplayPort
         unsigned getDscMaxSliceWidth();
         unsigned getDscDecoderColorDepthSupportMask();
         void setDscDecompressionDevice(bool bDscCapBasedOnParent);
+        virtual bool getDeviceSpecificData(NvU8 *oui, NvU8 *deviceIdString, 
+                                           NvU8 *hwRevision, NvU8 *swMajorRevision, 
+                                           NvU8 *swMinorRevision);
+
+        virtual bool setModeList(DisplayPort::DpModesetParams *pModeList, unsigned numModes);
     };
     class DeviceHDCPDetection : public Object, MessageManager::Message::MessageEventSink, Timer::TimerCallback
     {
