@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -33,10 +33,9 @@
 #include "dp_auxdefs.h"
 #include "dp_tracing.h"
 #include "dp_vrr.h"
+#include "dp_printf.h"
 #include <nvmisc.h>
 
-#include <ctrl/ctrl0073/ctrl0073dfp.h>
-#include <ctrl/ctrl0073/ctrl0073dp.h>
 #include <ctrl/ctrl0073/ctrl0073specific.h>
 #include <ctrl/ctrl0073/ctrl0073system.h>
 #include <ctrl/ctrl5070/ctrl5070or.h>
@@ -57,7 +56,7 @@ using namespace DisplayPort;
 // The first EvoMainLink constructor will populate that data base.
 // Later EvoMainLink will use values from that data base.
 //
-static struct DP_REGKEY_DATABASE dpRegkeyDatabase = {0};
+struct DP_REGKEY_DATABASE dpRegkeyDatabase = {0};
 
 enum DP_REG_VAL_TYPE
 {
@@ -74,7 +73,6 @@ const struct
     DP_REG_VAL_TYPE valueType;
 } DP_REGKEY_TABLE [] =
 {
-    {NV_DP_REGKEY_ENABLE_AUDIO_BEYOND_48K,          &dpRegkeyDatabase.bAudioBeyond48kEnabled,          DP_REG_VAL_BOOL},
     {NV_DP_REGKEY_OVERRIDE_DPCD_REV,                &dpRegkeyDatabase.dpcdRevOveride,                  DP_REG_VAL_U32},
     {NV_DP_REGKEY_DISABLE_SSC,                      &dpRegkeyDatabase.bSscDisabled,                    DP_REG_VAL_BOOL},
     {NV_DP_REGKEY_ENABLE_FAST_LINK_TRAINING,        &dpRegkeyDatabase.bFastLinkTrainingEnabled,        DP_REG_VAL_BOOL},
@@ -94,10 +92,13 @@ const struct
     {NV_DP_REGKEY_KEEP_OPT_LINK_ALIVE_SST,          &dpRegkeyDatabase.bOptLinkKeptAliveSst,            DP_REG_VAL_BOOL},
     {NV_DP_REGKEY_FORCE_EDP_ILR,                    &dpRegkeyDatabase.bBypassEDPRevCheck,              DP_REG_VAL_BOOL},
     {NV_DP_DSC_MST_CAP_BUG_3143315,                 &dpRegkeyDatabase.bDscMstCapBug3143315,            DP_REG_VAL_BOOL},
-    {NV_DP_REGKEY_ENABLE_OUI_RESTORING,             &dpRegkeyDatabase.bEnableOuiRestoring,             DP_REG_VAL_BOOL},
-    {NV_DP_CHECK_FEC_FOR_DDS_DSC_PANEL,             &dpRegkeyDatabase.bCheckFECForDynamicMuxDSCPanel,  DP_REG_VAL_BOOL},
     {NV_DP_REGKEY_POWER_DOWN_PHY,                   &dpRegkeyDatabase.bPowerDownPhyBeforeD3,           DP_REG_VAL_BOOL},
-    {NV_DP_REGKEY_REASSESS_MAX_LINK,                &dpRegkeyDatabase.bReassessMaxLink,                DP_REG_VAL_BOOL}
+    {NV_DP_REGKEY_REASSESS_MAX_LINK,                &dpRegkeyDatabase.bReassessMaxLink,                DP_REG_VAL_BOOL},
+    {NV_DP_REGKEY_MST_PCON_CAPS_READ_DISABLED,      &dpRegkeyDatabase.bMSTPCONCapsReadDisabled,        DP_REG_VAL_BOOL},
+    {NV_DP_REGKEY_FORCE_DSC_ON_SINK,                &dpRegkeyDatabase.bForceDscOnSink,                 DP_REG_VAL_BOOL},
+    {NV_DP_REGKEY_ENABLE_SKIP_DPCD_READS_WAR,       &dpRegkeyDatabase.bSkipFakeDeviceDpcdAccess,       DP_REG_VAL_BOOL},
+    {NV_DP_REGKEY_FLUSH_TIMESLOT_INFO_WHEN_DIRTY,   &dpRegkeyDatabase.bFlushTimeslotWhenDirty,         DP_REG_VAL_BOOL},
+    {NV_DP_REGKEY_DISABLE_TUNNEL_BW_ALLOCATION,     &dpRegkeyDatabase.bForceDisableTunnelBwAllocation, DP_REG_VAL_BOOL}
 };
 
 EvoMainLink::EvoMainLink(EvoInterface * provider, Timer * timer) :
@@ -115,13 +116,13 @@ EvoMainLink::EvoMainLink(EvoInterface * provider, Timer * timer) :
     this->initializeRegkeyDatabase();
     this->applyRegkeyOverrides();
 
-     _isDynamicMuxCapable       = false;
-     _isLTPhyRepeaterSupported  = true;
-     _rmPhyRepeaterCount        = 0;
-     dpMemZero(&_DSC, sizeof(_DSC));
-    queryGPUCapability();
-
-    queryAndUpdateDfpParams();
+    _isDynamicMuxCapable       = false;
+    _isMDMEnabled              = false;
+    _isLTPhyRepeaterSupported  = true;
+    _rmPhyRepeaterCount        = 0;
+    dpMemZero(&_DSC, sizeof(_DSC));
+    dpMemZero(&dfpParams, sizeof(dfpParams));
+    dpMemZero(&dpParams, sizeof(dpParams));
 
     //
     //  Tell RM to hands off on the DisplayPort hardware
@@ -262,63 +263,57 @@ NvU32 EvoMainLink::headToStream(NvU32 head, DP_SINGLE_HEAD_MULTI_STREAM_PIPELINE
     return streamIndex;
 }
 
-void EvoMainLink::queryGPUCapability()
+bool EvoMainLink::queryGPUCapability()
 {
-    NV0073_CTRL_CMD_DP_GET_CAPS_PARAMS params;
-
-    dpMemZero(&params, sizeof(params));
-    params.subDeviceInstance = subdeviceIndex;
-    params.sorIndex = provider->getSorIndex();
-    NvU32 code = provider->rmControl0073(NV0073_CTRL_CMD_DP_GET_CAPS, &params, sizeof(params));
+    dpMemZero(&dpParams, sizeof(dpParams));
+    dpParams.subDeviceInstance = subdeviceIndex;
+    dpParams.sorIndex = provider->getSorIndex();
+    NvU32 code = provider->rmControl0073(NV0073_CTRL_CMD_DP_GET_CAPS, &dpParams, sizeof(dpParams));
     if (code != NVOS_STATUS_SUCCESS)
     {
         DP_ASSERT(0 && "Unable to process GPU caps");
+        return false;
     }
+    //
+    // Check if MST feature needs to be disabled by regkey. This is requirement by few OEMs, they don't want to support
+    // MST feature on particular sku, whenever requested through INF.
+    //
+    _hasMultistream                 = (dpParams.bIsMultistreamSupported == NV_TRUE) && !_isMstDisabledByRegkey;
+    _isStreamCloningEnabled         = (dpParams.bIsSCEnabled == NV_TRUE) ? true : false;
+    _hasIncreasedWatermarkLimits    = (dpParams.bHasIncreasedWatermarkLimits == NV_TRUE) ? true : false;
+    _isFECSupported                 = (dpParams.bFECSupported == NV_TRUE) ? true : false;
+    _useDfpMaxLinkRateCaps          = (dpParams.bOverrideLinkBw == NV_TRUE) ? true : false;
+    _isLTPhyRepeaterSupported       = (dpParams.bIsTrainPhyRepeater == NV_TRUE) ? true : false;
+    _isDownspreadSupported          = (dpParams.bSupportDPDownSpread == NV_TRUE) ? true : false;
+
+    _gpuSupportedDpVersions         = dpParams.dpVersionsSupported;
+
+    if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _1_62, dpParams.maxLinkRate))
+        _maxLinkRateSupportedGpu = RBR; //in Hz
+    else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _2_70, dpParams.maxLinkRate))
+        _maxLinkRateSupportedGpu = HBR; //in Hz
+    else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _5_40, dpParams.maxLinkRate))
+        _maxLinkRateSupportedGpu = HBR2; //in Hz
+    else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _8_10, dpParams.maxLinkRate))
+        _maxLinkRateSupportedGpu = HBR3; //in Hz
     else
     {
-        //
-        // Check if MST feature needs to be disabled by regkey. This is requirement by few OEMs, they don't want to support
-        // MST feature on particular sku, whenever requested through INF.
-        //
-        _hasMultistream         = (params.bIsMultistreamSupported == NV_TRUE) && !_isMstDisabledByRegkey;
-
-        _gpuSupportedDpVersions = params.dpVersionsSupported;
-
-        _isStreamCloningEnabled = (params.bIsSCEnabled == NV_TRUE) ? true : false;
-        _hasIncreasedWatermarkLimits     = (params.bHasIncreasedWatermarkLimits == NV_TRUE) ? true : false;
-
-        _isFECSupported         = (params.bFECSupported == NV_TRUE) ? true : false;
-
-        _useDfpMaxLinkRateCaps  = (params.bOverrideLinkBw == NV_TRUE) ? true : false;
-
-        _isLTPhyRepeaterSupported        = (params.bIsTrainPhyRepeater == NV_TRUE) ? true : false;
-
-        if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _1_62, params.maxLinkRate))
-            _maxLinkRateSupportedGpu = RBR; //in Hz
-        else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _2_70, params.maxLinkRate))
-            _maxLinkRateSupportedGpu = HBR; //in Hz
-        else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _5_40, params.maxLinkRate))
-            _maxLinkRateSupportedGpu = HBR2; //in Hz
-        else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _8_10, params.maxLinkRate))
-            _maxLinkRateSupportedGpu = HBR3; //in Hz
-        else
-        {
-            DP_ASSERT(0 && "Unable to get max link rate");
-            // Assume that we can at least support RBR.
-            _maxLinkRateSupportedGpu = RBR;
-        }
-
-        if (!_isDscDisabledByRegkey)
-        {
-            _DSC.isDscSupported = params.DSC.bDscSupported ? true : false;
-            _DSC.encoderColorFormatMask = params.DSC.encoderColorFormatMask;
-            _DSC.lineBufferSizeKB = params.DSC.lineBufferSizeKB;
-            _DSC.rateBufferSizeKB = params.DSC.rateBufferSizeKB;
-            _DSC.bitsPerPixelPrecision = params.DSC.bitsPerPixelPrecision;
-            _DSC.maxNumHztSlices = params.DSC.maxNumHztSlices;
-            _DSC.lineBufferBitDepth = params.DSC.lineBufferBitDepth;
-        }
+        DP_ASSERT(0 && "Unable to get max link rate");
+        // Assume that we can at least support RBR.
+        _maxLinkRateSupportedGpu = RBR;
     }
+
+    if (!_isDscDisabledByRegkey)
+    {
+        _DSC.isDscSupported = dpParams.DSC.bDscSupported ? true : false;
+        _DSC.encoderColorFormatMask = dpParams.DSC.encoderColorFormatMask;
+        _DSC.lineBufferSizeKB = dpParams.DSC.lineBufferSizeKB;
+        _DSC.rateBufferSizeKB = dpParams.DSC.rateBufferSizeKB;
+        _DSC.bitsPerPixelPrecision = dpParams.DSC.bitsPerPixelPrecision;
+        _DSC.maxNumHztSlices = dpParams.DSC.maxNumHztSlices;
+        _DSC.lineBufferBitDepth = dpParams.DSC.lineBufferBitDepth;
+    }
+    return true;
 }
 
 void EvoMainLink::triggerACT()
@@ -621,7 +616,7 @@ void EvoMainLink::clearFlushMode(unsigned headMask, bool testMode)
     NvU32 ret = provider->rmControl5070(NV5070_CTRL_CMD_SET_SOR_FLUSH_MODE, &params, sizeof params);
     if (ret != NVOS_STATUS_SUCCESS)
     {
-        DP_LOG(("DP_EVO> Disabling flush mode failed!"));
+        DP_PRINTF(DP_ERROR, "DP_EVO> Disabling flush mode failed!");
     }
 }
 
@@ -648,18 +643,18 @@ bool EvoMainLink::physicalLayerSetTestPattern(PatternInfo * patternInfo)
         {
             ctrlPattern.testPattern = NV0073_CTRL_DP_TESTPATTERN_DATA_CSTM;
 
-            params.cstm.lower = patternInfo->ctsmLower;
-            params.cstm.middle = patternInfo->ctsmMiddle;
-            params.cstm.upper = patternInfo->ctsmUpper;
+            params.cstm.field_31_0 = patternInfo->ctsmLower;
+            params.cstm.field_63_32 = patternInfo->ctsmMiddle;
+            params.cstm.field_95_64 = patternInfo->ctsmUpper;
             break;
         }
 #ifdef NV0073_CTRL_DP_TESTPATTERN_DATA_HBR2COMPLIANCE
         case LINK_QUAL_HBR2_COMPLIANCE_EYE:
         {
             ctrlPattern.testPattern = NV0073_CTRL_DP_TESTPATTERN_DATA_HBR2COMPLIANCE;
-            params.cstm.lower = 0;
-            params.cstm.middle = 0;
-            params.cstm.upper = 0;
+            params.cstm.field_31_0 = 0;
+            params.cstm.field_63_32 = 0;
+            params.cstm.field_95_64 = 0;
             break;
         }
 #endif
@@ -732,7 +727,7 @@ AuxBus::status EvoAuxBus::transaction(Action action, Type type, int address,
     //
     if ((sizeRequested == 0) && (type & (i2cMot | i2c)) && (action == write))
     {
-        DP_LOG(("DP> Client requested address-only transaction"));
+        DP_PRINTF(DP_NOTICE, "DP> Client requested address-only transaction");
         params.bAddrOnly = NV_TRUE;
     }
     else if ((sizeRequested == 0) && (type == native))
@@ -783,7 +778,7 @@ AuxBus::status EvoAuxBus::transaction(Action action, Type type, int address,
         // error, the request still went out on the DPAUX channel as part of
         // the last IC-over-AUX write transaction. So the error should be ignored.
         //
-        DP_LOG(("DP> %s: Ignore ERROR_NOT_SUPPORTED for writeStatusUpdateRequest. Returning Success", __FUNCTION__));
+        DP_PRINTF(DP_NOTICE, "DP> %s: Ignore ERROR_NOT_SUPPORTED for writeStatusUpdateRequest. Returning Success", __FUNCTION__);
         return AuxBus::success;
     }
 
@@ -792,7 +787,7 @@ AuxBus::status EvoAuxBus::transaction(Action action, Type type, int address,
     {
         if (devicePlugged)
         {
-            DP_LOG(("DP> AuxChCtl Failing, if a device is connected you shouldn't be seeing this"));
+            DP_PRINTF(DP_ERROR, "DP> AuxChCtl Failing, if a device is connected you shouldn't be seeing this");
         }
         return nack;
     }
@@ -861,8 +856,10 @@ void EvoMainLink::postLinkTraining(NvU32 head)
 void EvoMainLink::initializeRegkeyDatabase()
 {
     NvU32 i;
+
     if (dpRegkeyDatabase.bInitialized)
         return;
+
     for (i = 0; i < sizeof(DP_REGKEY_TABLE)/sizeof(DP_REGKEY_TABLE[0]); i++)
     {
         NvU32 tempValue = 0;
@@ -883,6 +880,7 @@ void EvoMainLink::initializeRegkeyDatabase()
                 break;
         }
     }
+
     dpRegkeyDatabase.bInitialized = true;
 }
 
@@ -898,7 +896,7 @@ void EvoMainLink::applyRegkeyOverrides()
     _skipPowerdownEDPPanelWhenHeadDetach = dpRegkeyDatabase.bPoweroffEdpInHeadDetachSkipped;
     _applyLinkBwOverrideWarRegVal        = dpRegkeyDatabase.bLinkBwOverrideWarApplied;
     _enableMSAOverrideOverMST            = dpRegkeyDatabase.bMsaOverMstEnabled;
-    _enableFecCheckForDDS                = dpRegkeyDatabase.bCheckFECForDynamicMuxDSCPanel;
+    _isMSTPCONCapsReadDisabled           = dpRegkeyDatabase.bMSTPCONCapsReadDisabled;
 }
 
 NvU32 EvoMainLink::getRegkeyValue(const char *key)
@@ -980,25 +978,25 @@ bool EvoMainLink::train(const LinkConfiguration & link, bool force,
         DRF_DEF(0073_CTRL, _DP_CMD, _SET_LINK_BW,    _TRUE);
 
     if (link.multistream)
-        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _SET_FORMAT_MODE, _MULTI_STREAM );
+        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _SET_FORMAT_MODE, _MULTI_STREAM);
 
     if(link.bEnableFEC)
         dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _ENABLE_FEC, _TRUE);
 
     if (isPostLtAdjRequestGranted)
-        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _POST_LT_ADJ_REQ_GRANTED, _YES );
+        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _POST_LT_ADJ_REQ_GRANTED, _YES);
 
     if (link.enhancedFraming)
-        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _SET_ENHANCED_FRAMING, _TRUE );
+        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _SET_ENHANCED_FRAMING, _TRUE);
     if (bSkipLt)
-        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _SKIP_HW_PROGRAMMING, _YES );
+        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _SKIP_HW_PROGRAMMING, _YES);
     if (force)
-        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _FAKE_LINK_TRAINING, _DONOT_TOGGLE_TRANSMISSION );
+        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _FAKE_LINK_TRAINING, _DONOT_TOGGLE_TRANSMISSION);
 
     if (linkTrainingType == NO_LINK_TRAINING)
-        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _NO_LINK_TRAINING, _YES );
+        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _NO_LINK_TRAINING, _YES);
     else if (linkTrainingType == FAST_LINK_TRAINING)
-        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _FAST_LINK_TRAINING, _YES );
+        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _FAST_LINK_TRAINING, _YES);
 
     targetIndex = NV0073_CTRL_DP_DATA_TARGET_SINK;
     if (bTrainPhyRepeater && (_rmPhyRepeaterCount != phyRepeaterCount))
@@ -1009,14 +1007,20 @@ bool EvoMainLink::train(const LinkConfiguration & link, bool force,
 
     if (bTrainPhyRepeater)
     {
-
-        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _TRAIN_PHY_REPEATER, _YES );
+        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _TRAIN_PHY_REPEATER, _YES);
         //
         // Start from the one closest to GPU. Note this is 1-based index.
         //
         targetIndex = phyRepeaterCount;
     }
 
+    if (!this->isDownspreadSupported())
+    {
+        // If GPU does not support downspread, disabling downspread.
+        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _USE_DOWNSPREAD_SETTING, _FORCE);
+        dpCtrlCmd |= DRF_DEF(0073_CTRL, _DP_CMD, _DISABLE_DOWNSPREAD, _TRUE);
+    }
+
     NV_DPTRACE_INFO(LINK_TRAINING_START, link.multistream, link.peakRate, link.lanes,
                         phyRepeaterCount, _rmPhyRepeaterCount, bTrainPhyRepeater, targetIndex);
 
@@ -1050,7 +1054,6 @@ bool EvoMainLink::train(const LinkConfiguration & link, bool force,
             case EDP_3_24GHZ:
             case EDP_4_32GHZ:
             case HBR2:
-            case EDP_6_75GHZ:
             case HBR3:
                 linkBw = linkrate / DP_LINK_BW_FREQ_MULTI_MBPS;
                 dpCtrlData = FLD_SET_DRF_NUM(0073_CTRL, _DP_DATA, _SET_LINK_BW,
@@ -1165,8 +1168,12 @@ bool EvoMainLink::train(const LinkConfiguration & link, bool force,
                 // 1. CR or EQ phase failed.
                 // 2. The request link bandwidth is NOT RBR
                 //
+                if (!requestRmLC.lowerConfig())
+                {
+                    // If no valid link config could be found, break here.
+                    break;
+                }
                 fallback = true;
-                requestRmLC.lowerConfig();
             }
             else
             {
@@ -1238,14 +1245,10 @@ bool EvoMainLink::train(const LinkConfiguration & link, bool force,
     retLink->setLaneRate(requestRmLC.peakRate, result ? requestRmLC.lanes : 0);
     retLink->setLTCounter(ltCounter);
 
-    // For release branch only, check FEC return values and update to "retLink"
-    if (_enableFecCheckForDDS)
+    if (requestRmLC.bEnableFEC && (FLD_TEST_DRF(0073_CTRL_DP, _ERR, _ENABLE_FEC, _ERR, err)))
     {
-        if (requestRmLC.bEnableFEC && (FLD_TEST_DRF(0073_CTRL_DP, _ERR, _ENABLE_FEC, _ERR, err)))
-        {
-            retLink->bEnableFEC = false;
-            DP_ASSERT(0);
-        }
+        retLink->bEnableFEC = false;
+        DP_ASSERT(0);
     }
 
     NV_DPTRACE_INFO(LINK_TRAINING_DONE, status, requestRmLC.peakRate, requestRmLC.lanes);
@@ -1347,7 +1350,16 @@ void EvoMainLink::getLinkConfig(unsigned &laneCount, NvU64 & linkRate)
     if (code == NVOS_STATUS_SUCCESS)
     {
         laneCount = params.laneCount;
-        linkRate = (NvU64)27000000 * params.linkBW;    // BUG: Beware, turbo mode need to be taken into account
+
+        if (params.linkBW != 0)
+        {
+            linkRate = ((NvU64)params.linkBW) * DP_LINK_BW_FREQ_MULTI_MBPS;
+        }
+        else
+        {
+            // No link rate available.
+            linkRate = 0;
+        }
     }
     else
     {
@@ -1370,7 +1382,6 @@ bool EvoMainLink::getMaxLinkConfigFromUefi(NvU8 &linkRate, NvU8 &laneCount)
 
 bool EvoMainLink::queryAndUpdateDfpParams()
 {
-    NV0073_CTRL_DFP_GET_INFO_PARAMS  dfpParams;
     NvU32 dfpFlags;
     dpMemZero(&dfpParams, sizeof(dfpParams));
     dfpParams.subDeviceInstance = subdeviceIndex;
@@ -1420,7 +1431,9 @@ bool EvoMainLink::queryAndUpdateDfpParams()
             break;
     }
 
+
     _isDynamicMuxCapable = FLD_TEST_DRF(0073, _CTRL_DFP_FLAGS, _DYNAMIC_MUX_CAPABLE, _TRUE, dfpFlags);
+    _isMDMEnabled = FLD_TEST_DRF(0073, _CTRL_DFP_FLAGS, _MDM, _ENABLED, dfpFlags);
 
     return true;
 }
@@ -1512,6 +1525,10 @@ bool EvoMainLink::skipPowerdownEdpPanelWhenHeadDetach()
     return _skipPowerdownEDPPanelWhenHeadDetach;
 }
 
+bool EvoMainLink::isMSTPCONCapsReadDisabled()
+{
+    return _isMSTPCONCapsReadDisabled;
+}
 
 bool EvoMainLink::isActive()
 {
@@ -1713,7 +1730,9 @@ void EvoMainLink::configureTriggerAll(NvU32 head, bool enable)
 
 MainLink * DisplayPort::MakeEvoMainLink(EvoInterface * provider, Timer * timer)
 {
-    return new EvoMainLink(provider, timer);
+    MainLink *main;
+    main = new EvoMainLink(provider, timer);
+    return main;
 }
 
 AuxBus   * DisplayPort::MakeEvoAuxBus(EvoInterface * provider, Timer * timer)
@@ -1745,7 +1764,7 @@ bool EvoMainLink::dscCrcTransaction(NvBool bEnable, gpuDscCrc *data, NvU16 *head
     code = provider->rmControl0073(NV0073_CTRL_CMD_DFP_DSC_CRC_CONTROL, &params, sizeof(params));
     if (code != NVOS_STATUS_SUCCESS)
     {
-        DP_LOG(("DP> Crc control failed."));
+        DP_PRINTF(DP_ERROR, "DP> Crc control failed.");
         return false;
     }
 
@@ -1772,8 +1791,8 @@ bool EvoMainLink::dscCrcTransaction(NvBool bEnable, gpuDscCrc *data, NvU16 *head
 //
 bool EvoMainLink::configureLinkRateTable
 (
-    const NvU16 *pLinkRateTable,
-    LinkRates   *pLinkRates
+    const NvU16     *pLinkRateTable,
+    LinkRates       *pLinkRates
 )
 {
     NV0073_CTRL_CMD_DP_CONFIG_INDEXED_LINK_RATES_PARAMS params;
@@ -1810,14 +1829,12 @@ bool EvoMainLink::configureLinkRateTable
                 case linkBW_3_24Gbps:
                 case linkBW_4_32Gbps:
                 case linkBW_5_40Gbps:
-                case linkBW_6_75Gbps:
                 case linkBW_8_10Gbps:
-                    pLinkRates->import(params.linkBwTbl[i]);
+                    pLinkRates->import((NvU8)params.linkBwTbl[i]);
                     break;
-
                 default:
-                    DP_LOG(("DP_EVO> %s: Unsupported link rate received",
-                           __FUNCTION__));
+                    DP_PRINTF(DP_ERROR, "DP_EVO> %s: Unsupported link rate received",
+                             __FUNCTION__);
                     DP_ASSERT(0);
                     break;
             }
