@@ -38,9 +38,7 @@
 #include "nvswitch/ls10/dev_nvlsaw_ip.h"
 #include "nvswitch/ls10/dev_nvlsaw_ip_addendum.h"
 #include "nvswitch/ls10/dev_riscv_pri.h"
-
 #include "nvswitch/ls10/dev_nport_ip.h"
-#include "nvswitch/ls10/dev_npg_ip.h"
 
 #include "flcn/flcnable_nvswitch.h"
 #include "flcn/flcn_nvswitch.h"
@@ -182,7 +180,7 @@ dumpDebugRegisters
     NVSWITCH_PRINT(device, ERROR, "RESET_PLM             : 0x%08x\n", regRESET_PLM);
     NVSWITCH_PRINT(device, ERROR, "EXE_PLM               : 0x%08x\n", regEXE_PLM);
 }
-#endif // defined(DEVELOP) || defined(DEBUG) || defined(NV_MODS)
+#endif  // defined(DEVELOP) || defined(DEBUG) || defined(NV_MODS)
 
 /*
  * @Brief : Attach or Detach driver to SOE Queues
@@ -226,13 +224,20 @@ _nvswitch_is_soe_attached_ls10
 )
 {
     NvU32 val;
+    NvBool bSoeAttached;
 
     val = NVSWITCH_SAW_RD32_LS10(device, _NVLSAW, _SOE_ATTACH_DETACH);
+    bSoeAttached = FLD_TEST_DRF(_NVLSAW, _SOE_ATTACH_DETACH, _STATUS, _ATTACHED, val);
 
-    return FLD_TEST_DRF(_NVLSAW, _SOE_ATTACH_DETACH, _STATUS, _ATTACHED, val);
+    return bSoeAttached;
 }
 
-// BACK UP Nport state and reset NPORT
+/*
+ * @Brief : Backup NPORT state and issue NPORT reset
+ *
+ * @param[in] device
+ * @param[in] nport
+ */
 NvlStatus
 nvswitch_soe_issue_nport_reset_ls10
 (
@@ -240,7 +245,7 @@ nvswitch_soe_issue_nport_reset_ls10
     NvU32 nport
 )
 {
-    FLCN *pFlcn       = device->pSoe->pFlcn;
+    FLCN *pFlcn = device->pSoe->pFlcn;
     NvU32               cmdSeqDesc = 0;
     NV_STATUS           status;
     RM_FLCN_CMD_SOE     cmd;
@@ -256,7 +261,7 @@ nvswitch_soe_issue_nport_reset_ls10
     pNportReset->nport = nport;
     pNportReset->cmdType = RM_SOE_CORE_CMD_ISSUE_NPORT_RESET;
 
-    nvswitch_timeout_create(NVSWITCH_INTERVAL_1SEC_IN_NS * 5, &timeout);
+    nvswitch_timeout_create(NVSWITCH_INTERVAL_5MSEC_IN_NS, &timeout);
     status = flcnQueueCmdPostBlocking(device, pFlcn,
                                       (PRM_FLCN_CMD)&cmd,
                                       NULL,                 // pMsg
@@ -274,7 +279,12 @@ nvswitch_soe_issue_nport_reset_ls10
     return NVL_SUCCESS;
 }
 
-// De-reset NPORT and restore NPORT state
+/*
+ * @Brief : De-Assert NPORT reset and restore NPORT state
+ *
+ * @param[in] device
+ * @param[in] nport
+ */
 NvlStatus
 nvswitch_soe_restore_nport_state_ls10
 (
@@ -292,13 +302,13 @@ nvswitch_soe_restore_nport_state_ls10
     nvswitch_os_memset(&cmd, 0, sizeof(cmd));
 
     cmd.hdr.unitId = RM_SOE_UNIT_CORE;
-    cmd.hdr.size   = sizeof(cmd);
+    cmd.hdr.size   = RM_SOE_CMD_SIZE(CORE, NPORT_STATE);
 
     pNportState = &cmd.cmd.core.nportState;
     pNportState->nport = nport;
     pNportState->cmdType = RM_SOE_CORE_CMD_RESTORE_NPORT_STATE;
 
-    nvswitch_timeout_create(NVSWITCH_INTERVAL_1SEC_IN_NS * 5, &timeout);
+    nvswitch_timeout_create(NVSWITCH_INTERVAL_5MSEC_IN_NS, &timeout);
     status = flcnQueueCmdPostBlocking(device, pFlcn,
                                       (PRM_FLCN_CMD)&cmd,
                                       NULL,                 // pMsg
@@ -346,7 +356,7 @@ nvswitch_set_nport_tprod_state_ls10
     nvswitch_os_memset(&cmd, 0, sizeof(cmd));
 
     cmd.hdr.unitId = RM_SOE_UNIT_CORE;
-    cmd.hdr.size   = RM_SOE_CMD_SIZE(CORE, NPORT_STATE);
+    cmd.hdr.size   = RM_SOE_CMD_SIZE(CORE, NPORT_TPROD_STATE);
 
     nportTprodState = &cmd.cmd.core.nportTprodState;
     nportTprodState->nport = nport;
@@ -391,7 +401,7 @@ nvswitch_soe_init_l2_state_ls10
 
     if (!nvswitch_is_soe_supported(device))
     {
-        NVSWITCH_PRINT(device, INFO, "%s: SOE is not supported. skipping!\n",
+        NVSWITCH_PRINT(device, INFO, "%s: SOE is not supported\n",
                        __FUNCTION__);
         return;
     }
@@ -400,7 +410,7 @@ nvswitch_soe_init_l2_state_ls10
 
     nvswitch_os_memset(&cmd, 0, sizeof(cmd));
     cmd.hdr.unitId = RM_SOE_UNIT_CORE;
-    cmd.hdr.size   = sizeof(cmd);
+    cmd.hdr.size   = RM_SOE_CMD_SIZE(CORE, L2_STATE);
 
     pL2State = &cmd.cmd.core.l2State;
     pL2State->cmdType = RM_SOE_CORE_CMD_INIT_L2_STATE;
@@ -420,6 +430,242 @@ nvswitch_soe_init_l2_state_ls10
     }
 }
 
+/*
+ * @Brief : Enable/Disable NPORT interrupts
+ *
+ * @param[in] device
+ * @param[in] nport
+ */
+NvlStatus
+nvswitch_soe_set_nport_interrupts_ls10
+(
+    nvswitch_device *device,
+    NvU32           nport,
+    NvBool          bEnable
+)
+{
+    FLCN            *pFlcn;
+    NvU32            cmdSeqDesc = 0;
+    NV_STATUS        status;
+    RM_FLCN_CMD_SOE  cmd;
+    NVSWITCH_TIMEOUT timeout;
+    RM_SOE_CORE_CMD_NPORT_INTRS *pNportIntrs;
+
+    if (!nvswitch_is_soe_supported(device))
+    {
+        NVSWITCH_PRINT(device, ERROR,
+            "%s: SOE is not supported\n",
+            __FUNCTION__);
+        return -NVL_ERR_INVALID_STATE;
+    }
+
+    pFlcn       = device->pSoe->pFlcn;
+
+    nvswitch_os_memset(&cmd, 0, sizeof(cmd));
+    cmd.hdr.unitId = RM_SOE_UNIT_CORE;
+    cmd.hdr.size   = RM_SOE_CMD_SIZE(CORE, NPORT_INTRS);
+
+    pNportIntrs = &cmd.cmd.core.nportIntrs;
+    pNportIntrs->cmdType = RM_SOE_CORE_CMD_SET_NPORT_INTRS;
+    pNportIntrs->nport = nport;
+    pNportIntrs->bEnable = bEnable;
+
+    nvswitch_timeout_create(NVSWITCH_INTERVAL_5MSEC_IN_NS, &timeout);
+    status = flcnQueueCmdPostBlocking(device, pFlcn,
+                                      (PRM_FLCN_CMD)&cmd,
+                                      NULL,                 // pMsg
+                                      NULL,                 // pPayload
+                                      SOE_RM_CMDQ_LOG_ID,
+                                      &cmdSeqDesc,
+                                      &timeout);
+    if (status != NV_OK)
+    {
+        NVSWITCH_PRINT(device, ERROR,
+            "%s: Failed to send SET_NPORT_INTRS command to SOE, status 0x%x\n", 
+            __FUNCTION__, status);
+        return -NVL_ERR_GENERIC;
+    }
+
+    return NVL_SUCCESS;
+}
+
+/*
+ * @Brief : Disable NPORT Fatal Interrupt in SOE
+ *
+ * @param[in] device
+ * @param[in] nport
+ * @param[in] nportIntrEnable
+ * @param[in] nportIntrType
+ */
+void
+nvswitch_soe_disable_nport_fatal_interrupts_ls10
+(
+    nvswitch_device *device,
+    NvU32 nport,
+    NvU32 nportIntrEnable,
+    NvU8  nportIntrType   
+)
+{
+    FLCN            *pFlcn;
+    NvU32            cmdSeqDesc = 0;
+    NV_STATUS        status;
+    RM_FLCN_CMD_SOE  cmd;
+    NVSWITCH_TIMEOUT timeout;
+    RM_SOE_CORE_CMD_NPORT_FATAL_INTR *pNportIntrDisable;
+    NVSWITCH_GET_BIOS_INFO_PARAMS p = { 0 };
+
+    status = device->hal.nvswitch_ctrl_get_bios_info(device, &p);
+    if ((status != NVL_SUCCESS) || ((p.version & SOE_VBIOS_VERSION_MASK) < 
+            SOE_VBIOS_REVLOCK_DISABLE_NPORT_FATAL_INTR))
+    {
+        NVSWITCH_PRINT(device, INFO,
+            "%s: Skipping DISABLE_NPORT_FATAL_INTR command to SOE.  Update firmware "
+            "from .%02X to .%02X\n",
+            __FUNCTION__, (NvU32)((p.version & SOE_VBIOS_VERSION_MASK) >> 16), 
+            SOE_VBIOS_REVLOCK_DISABLE_NPORT_FATAL_INTR);
+        return;
+    }
+
+    if (!nvswitch_is_soe_supported(device))
+    {
+        NVSWITCH_PRINT(device, INFO,
+            "%s: SOE is not supported\n",
+            __FUNCTION__);
+        return;
+    }
+
+    pFlcn = device->pSoe->pFlcn;
+
+    nvswitch_os_memset(&cmd, 0, sizeof(cmd));
+    cmd.hdr.unitId = RM_SOE_UNIT_CORE;
+    cmd.hdr.size   = RM_SOE_CMD_SIZE(CORE, NPORT_FATAL_INTR);
+
+    pNportIntrDisable = &cmd.cmd.core.nportDisableIntr;
+    pNportIntrDisable->cmdType = RM_SOE_CORE_CMD_DISABLE_NPORT_FATAL_INTR;
+    pNportIntrDisable->nport   = nport;
+    pNportIntrDisable->nportIntrEnable = nportIntrEnable;
+    pNportIntrDisable->nportIntrType = nportIntrType;
+
+    nvswitch_timeout_create(NVSWITCH_INTERVAL_5MSEC_IN_NS, &timeout);
+    status = flcnQueueCmdPostBlocking(device, pFlcn,
+                                      (PRM_FLCN_CMD)&cmd,
+                                      NULL,                 // pMsg
+                                      NULL,                 // pPayload
+                                      SOE_RM_CMDQ_LOG_ID,
+                                      &cmdSeqDesc,
+                                      &timeout);
+    if (status != NV_OK)
+    {
+        NVSWITCH_PRINT(device, ERROR,
+            "%s: Failed to send DISABLE_NPORT_FATAL_INTR command to SOE, status 0x%x\n", 
+            __FUNCTION__, status);
+    }
+}
+
+/*
+ * @Brief : Issue INGRESS STOP in SOE
+ *
+ * @param[in] device
+ * @param[in] nport
+ * @param[in] bStop
+ */
+NvlStatus
+nvswitch_soe_issue_ingress_stop_ls10
+(
+    nvswitch_device *device,
+    NvU32 nport,
+    NvBool bStop
+)
+{
+    FLCN *pFlcn;
+    NvU32               cmdSeqDesc = 0;
+    NV_STATUS           status;
+    RM_FLCN_CMD_SOE     cmd;
+    NVSWITCH_TIMEOUT    timeout;
+    RM_SOE_CORE_CMD_INGRESS_STOP *pIngressStop;
+    NVSWITCH_GET_BIOS_INFO_PARAMS params = { 0 };
+    NvBool bKeepPolling;
+    NvU32 val;
+
+    if (!nvswitch_is_soe_supported(device))
+    {
+        NVSWITCH_PRINT(device, INFO,
+            "%s: SOE is not supported\n",
+            __FUNCTION__);
+        return NVL_SUCCESS; // -NVL_ERR_NOT_SUPPORTED
+    }
+
+    status = device->hal.nvswitch_ctrl_get_bios_info(device, &params);
+    if ((status != NVL_SUCCESS) || ((params.version & SOE_VBIOS_VERSION_MASK) < 
+            SOE_VBIOS_REVLOCK_ISSUE_INGRESS_STOP))
+    {
+        return NVL_SUCCESS; // -NVL_ERR_NOT_SUPPORTED
+    }
+
+    pFlcn = device->pSoe->pFlcn;
+     
+    nvswitch_os_memset(&cmd, 0, sizeof(cmd));
+
+    cmd.hdr.unitId = RM_SOE_UNIT_CORE;
+    cmd.hdr.size   = RM_SOE_CMD_SIZE(CORE, INGRESS_STOP);
+     
+    pIngressStop = &cmd.cmd.core.ingressStop;
+    pIngressStop->nport = nport;
+    pIngressStop->cmdType = RM_SOE_CORE_CMD_ISSUE_INGRESS_STOP;
+    pIngressStop->bStop = bStop;
+     
+    nvswitch_timeout_create(NVSWITCH_INTERVAL_5MSEC_IN_NS, &timeout);
+    status = flcnQueueCmdPostBlocking(device, pFlcn,
+                                      (PRM_FLCN_CMD)&cmd,
+                                      NULL,                 // pMsg
+                                      NULL,                 // pPayload
+                                      SOE_RM_CMDQ_LOG_ID,
+                                      &cmdSeqDesc,
+                                      &timeout);
+    if (status != NV_OK)
+    {
+        NVSWITCH_PRINT(device, ERROR,
+            "%s: Failed to send INGRESS STOP command to SOE status 0x%x\n", 
+            __FUNCTION__, status);
+        return -NVL_ERR_GENERIC;
+    }
+
+    //
+    // After asserting INGRESS_STOP, wait until nport is idle
+    // to indicate that traffic is drained out.
+    //
+    if (bStop)
+    {
+        nvswitch_timeout_create(NVSWITCH_INTERVAL_1MSEC_IN_NS, &timeout);
+        do
+        {
+            bKeepPolling = (nvswitch_timeout_check(&timeout)) ? NV_FALSE : NV_TRUE;
+
+            val = NVSWITCH_NPORT_RD32_LS10(device, nport, _NPORT, _STATUS);
+
+            if (FLD_TEST_DRF(_NPORT, _STATUS, _INGRESS_IDLE, _IDLE, val) &&
+                FLD_TEST_DRF(_NPORT, _STATUS, _ROUTE_IDLE,   _IDLE, val))
+            {
+                break;
+            }
+
+            NVSWITCH_NSEC_DELAY(2 * NVSWITCH_INTERVAL_1USEC_IN_NS);
+
+        } while (bKeepPolling);
+
+        if (!FLD_TEST_DRF(_NPORT, _STATUS, _INGRESS_IDLE, _IDLE, val) ||
+            !FLD_TEST_DRF(_NPORT, _STATUS, _ROUTE_IDLE,   _IDLE, val))
+        {
+            NVSWITCH_PRINT(device, ERROR,
+                "%s: Traffic failed to drain after ingress stop is asserted, val = 0x%x\n", 
+                __FUNCTION__, val);
+            return -NVL_ERR_GENERIC;
+        }
+    }
+
+    return NVL_SUCCESS;
+}
+
 /*
  * @Brief : Init sequence for SOE FSP RISCV image
  *
@@ -521,7 +767,6 @@ nvswitch_unload_soe_ls10
     // Detach driver from SOE Queues
     _nvswitch_soe_attach_detach_driver_ls10(device, NV_FALSE);
 
-
     return NVL_SUCCESS;
 }
 
@@ -557,6 +802,40 @@ nvswitch_soe_register_event_callbacks_ls10
         return -NVL_ERR_INVALID_STATE;
     }
 
+    // Register CCI callback funcion
+    status = flcnQueueEventRegister(
+                device, pFlcn,
+                RM_SOE_UNIT_CCI,
+                NULL,
+                nvswitch_cci_soe_callback_ls10,
+                NULL,
+                &pSoe->cciEvtDesc);
+
+    if (status != NV_OK)
+    {
+        NVSWITCH_PRINT(device, ERROR,
+            "%s: Failed to register CCI event handler.\n",
+            __FUNCTION__);
+        return -NVL_ERR_INVALID_STATE;
+    }
+
+    // Register Heartbeat callback funcion
+    status = flcnQueueEventRegister(
+                device, pFlcn,
+                RM_SOE_UNIT_HEARTBEAT,
+                NULL,
+                nvswitch_heartbeat_soe_callback_ls10,
+                NULL,
+                &pSoe->heartbeatEvtDesc);
+
+    if (status != NV_OK)
+    {
+        NVSWITCH_PRINT(device, ERROR,
+            "%s: Failed to register Heartbeat event handler.\n",
+            __FUNCTION__);
+        return -NVL_ERR_INVALID_STATE;
+    }
+
     return NVL_SUCCESS;
 }
 
@@ -579,6 +858,25 @@ nvswitch_soe_unregister_events_ls10
             "%s: Failed to un-register thermal event handler.\n",
             __FUNCTION__);
     }
+    // un-register thermal callback funcion
+    status = flcnQueueEventUnregister(device, pFlcn,
+                                      pSoe->cciEvtDesc);
+    if (status != NV_OK)
+    {
+        NVSWITCH_PRINT(device, ERROR,
+            "%s: Failed to un-register cci event handler.\n",
+            __FUNCTION__);
+    }
+
+    // un-register heartbeat callback funcion
+    status = flcnQueueEventUnregister(device, pFlcn,
+                                      pSoe->heartbeatEvtDesc);
+    if (status != NV_OK)
+    {
+        NVSWITCH_PRINT(device, ERROR,
+            "%s: Failed to un-register heartbeat event handler.\n",
+            __FUNCTION__);
+    }
 }
 
 /*!
@@ -1201,4 +1499,3 @@ soeSetupHal_LS10
     pHal->waitForInitAck     = _soeWaitForInitAck_LS10;
     pHal->i2cAccess          = _soeI2CAccess_LS10;
 }
-
