@@ -64,6 +64,11 @@ NVSurfaceEvoPtr nvEvoGetSurfaceFromHandleNoDispHWAccessOk(
     const NVEvoApiHandlesRec *pOpenDevSurfaceHandles,
     NvKmsSurfaceHandle surfaceHandle);
 
+NVSurfaceEvoPtr nvEvoGetSurfaceFromHandleNoHWAccess(
+    const NVDevEvoRec *pDevEvo,
+    const NVEvoApiHandlesRec *pOpenDevSurfaceHandles,
+    NvKmsSurfaceHandle surfaceHandle);
+
 NVDeferredRequestFifoRec *nvEvoRegisterDeferredRequestFifo(
     NVDevEvoPtr pDevEvo,
     NVSurfaceEvoPtr pSurfaceEvo);
@@ -72,6 +77,22 @@ void nvEvoUnregisterDeferredRequestFifo(
     NVDevEvoPtr pDevEvo,
     NVDeferredRequestFifoRec *pDeferredRequestFifo);
 
+NVVblankSemControl *nvEvoEnableVblankSemControl(
+    NVDevEvoRec *pDevEvo,
+    NVDispEvoRec *pDispEvo,
+    NvU32 apiHeadMask,
+    NVSurfaceEvoRec *pSurfaceEvo,
+    NvU64 surfaceOffset);
+
+NvBool nvEvoDisableVblankSemControl(
+    NVDevEvoRec *pDevEvo,
+    NVVblankSemControl *pVblankSemControl);
+
+NvBool nvEvoAccelVblankSemControls(
+    NVDevEvoPtr pDevEvo,
+    NvU32 dispIndex,
+    NvU32 hwHeadMask);
+
 static inline NvBool nvEvoIsSurfaceOwner(const NVSurfaceEvoRec *pSurfaceEvo,
                                          const struct NvKmsPerOpenDev *pOpenDev,
                                          NvKmsSurfaceHandle surfaceHandle)
