@@ -30,15 +30,39 @@
 #include "class/clc86fsw.h"
 #include "ctrl/ctrl2080/ctrl2080internal.h"
 #include "nvrm_registry.h"
-#include "kernel/gpu/conf_compute/ccsl.h"
 
+static void initKeyRotationRegistryOverrides(OBJGPU *pGpu, ConfidentialCompute *pConfCompute);
 static void getKeyPairForKeySpace(NvU32 keySpace, NvBool bKernel, NvU32 *pGlobalH2DKey, NvU32 *pGlobalD2HKey);
 static NV_STATUS triggerKeyRotationByKeyPair(OBJGPU *pGpu, ConfidentialCompute *pConfCompute, NvU32 h2dKey, NvU32 d2hKey);
 static NV_STATUS calculateEncryptionStatsByKeyPair(OBJGPU *pGpu, ConfidentialCompute *pConfCompute, NvU32 h2dKey, NvU32 d2hKey);
+static NV_STATUS notifyKeyRotationByKeyPair(OBJGPU *pGpu, ConfidentialCompute *pConfCompute, NvU32 h2dKey);
 static NvBool isLowerThresholdCrossed(ConfidentialCompute *pConfCompute, NvU32 h2dKey, NvU32 d2hKey);
 static NvBool isUpperThresholdCrossed(ConfidentialCompute *pConfCompute, NvU32 h2dKey, NvU32 d2hKey);
 static NV_STATUS keyRotationTimeoutCallback(OBJGPU *pGpu, OBJTMR *pTmr, TMR_EVENT *pTmrEvent);
 
+/*!
+ * Conditionally enables key rotation support
+ *
+ * @param[in]     pGpu                     : OBJGPU Pointer
+ * @param[in]     pConfCompute             : ConfidentialCompute pointer
+ */
+NV_STATUS
+confComputeEnableKeyRotationSupport_GH100
+(
+    OBJGPU *pGpu,
+    ConfidentialCompute *pConfCompute
+)
+{
+
+    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_ENABLED) &&
+        pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_CC_FEATURE_ENABLED))
+    {
+        pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_SUPPORTED, NV_TRUE);
+        initKeyRotationRegistryOverrides(pGpu, pConfCompute);
+    }
+    return NV_OK;
+}
+
 /*!
  * Enables/disables key rotation by setting up the 1 sec callback for key rotation
  *
@@ -71,37 +95,6 @@ confComputeEnableKeyRotationCallback_GH100
     return NV_OK;
 }
 
-/*!
- * Returns H2D and D2H keys for given priv level.
- * For kernel keys, this will only return UVM keys and not the GSP or scrubber keys.
- *
- * @param[in]     pGpu                     : OBJGPU Pointer
- * @param[in]     pConfCompute             : ConfidentialCompute pointer
- * @param[in]     keySpace                 : keyspace
- * @param[in]     bKernel                  : is it kernel priv level
- * @param[out]    pGlobalH2DKey            : h2d key pointer
- * @param[out]    pGlobalD2HKey            : d2h key pointer
- */
-void
-confComputeGetKeyPairForKeySpace_GH100
-(
-    OBJGPU *pGpu,
-    ConfidentialCompute *pConfCompute,
-    NvU32 keySpace,
-    NvBool bKernel,
-    NvU32 *pGlobalH2DKey,
-    NvU32 *pGlobalD2HKey
-)
-{
-    NvU32 h2dKey, d2hKey;
-    getKeyPairForKeySpace(keySpace, bKernel, &h2dKey, &d2hKey);
-    if (pGlobalH2DKey != NULL)
-        *pGlobalH2DKey = h2dKey;
-
-    if (pGlobalD2HKey != NULL)
-        *pGlobalD2HKey = d2hKey;
-}
-
  /*!
   * Calculates encryption statistics and triggers key rotation if thresholds are crossed.
   *
@@ -129,8 +122,8 @@ confComputeTriggerKeyRotation_GH100
             continue;
         }
 
-        // calculate kernel channels stats for keyspace. (this doesn't include scrubber keys)
-        if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_KEY_ROTATION, _KERNEL_KEYS, _YES, pConfCompute->keyRotationEnableMask))
+        // calculate kernel channels stats for keyspace
+        if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_DUMMY_KEY_ROTATION, _KERNEL_KEYS, _YES, pConfCompute->keyRotationEnableMask))
         {
             getKeyPairForKeySpace(keySpace, NV_TRUE, &globalH2DKey, &globalD2HKey);
             tempStatus = triggerKeyRotationByKeyPair(pGpu, pConfCompute, globalH2DKey, globalD2HKey);
@@ -143,7 +136,7 @@ confComputeTriggerKeyRotation_GH100
         }
 
         // calculate user channels stats for keyspace
-        if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_KEY_ROTATION, _USER_KEYS, _YES, pConfCompute->keyRotationEnableMask))
+        if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_DUMMY_KEY_ROTATION, _USER_KEYS, _YES, pConfCompute->keyRotationEnableMask))
         {
             getKeyPairForKeySpace(keySpace, NV_FALSE, &globalH2DKey, &globalD2HKey);
             tempStatus = triggerKeyRotationByKeyPair(pGpu, pConfCompute, globalH2DKey, globalD2HKey);
@@ -174,6 +167,30 @@ triggerKeyRotationByKeyPair
     NV_ASSERT_OK_OR_RETURN(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, h2dKey, &h2dIndex));
     NV_ASSERT_OK_OR_RETURN(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, d2hKey, &d2hIndex));
 
+    // we won't need this once we have encryption statistics since unused keys will have stats = 0
+    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED))
+    {
+        CHANNEL_ITERATOR iter = {0};
+        KernelChannel *pKernelChannel = NULL;
+        NV_ASSERT_OK_OR_RETURN(confComputeInitChannelIterForKey(pGpu, pConfCompute, h2dKey, &iter));
+        //
+        // If there are no channels alive on a key then reset its dummy KR counter and bail out.
+        // Else increment its dummy KR counter and continue.
+        //
+        if (confComputeGetNextChannelForKey(pGpu, pConfCompute, &iter, h2dKey, &pKernelChannel) != NV_OK)
+        {
+            pConfCompute->keyRotationCallbackCount[h2dIndex] = 1;
+            pConfCompute->keyRotationCallbackCount[d2hIndex] = 1;
+            return NV_OK;
+        }
+        else
+        {
+            pConfCompute->keyRotationCallbackCount[h2dIndex]++;
+            pConfCompute->keyRotationCallbackCount[d2hIndex]++;
+            NV_PRINTF(LEVEL_ERROR, "DUMMY KR: COUNT = %d for key 0x%x\n", pConfCompute->keyRotationCallbackCount[h2dIndex], h2dKey);
+        }
+    }
+
     //
     // If key rotation is alredy scheduled because we crossed upper threshold or hit timeout
     // then we dont need to update encryption statistics as they will be zeroed out soon.
@@ -206,81 +223,36 @@ triggerKeyRotationByKeyPair
         {
             NV_ASSERT_OK_OR_RETURN(confComputeSetKeyRotationStatus(pConfCompute, h2dKey, KEY_ROTATION_STATUS_PENDING));
 
-            // Schedule first callback. Timeouts are not enforced on kernel channels (UVM).
-            if (!confComputeGlobalKeyIsUvmKey_HAL(pConfCompute, h2dKey))
+            //
+            // Start the timeout timer once lower threshold is crossed.
+            //
+            // If timer is not already created then create it now. Else, just schedule a callback.
+            // make sure callback is canceled if we schedule the KR task (after crossing lower or upper threshold)
+            // make sure all these timer events are deleted as part of RM shutdown
+            //
+            OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
+            if (pConfCompute->ppKeyRotationTimer[h2dIndex] == NULL)
             {
-                //
-                // Start the timeout timer once lower threshold is crossed.
-                //
-                // If timer is not already created then create it now. Else, just schedule a callback.
-                // make sure callback is canceled if we schedule the KR task (after crossing lower or upper threshold)
-                // make sure all these timer events are deleted as part of RM shutdown
-                //
-                OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
-                if (pConfCompute->keyRotationTimeoutInfo[h2dIndex].pTimer == NULL)
-                {
-                    NvU32 *pH2DKey = portMemAllocNonPaged(sizeof(NvU32));
-                    *pH2DKey = h2dKey;
-                    NV_ASSERT_OK_OR_RETURN(tmrEventCreate(pTmr, &pConfCompute->keyRotationTimeoutInfo[h2dIndex].pTimer,
-                                                          keyRotationTimeoutCallback, (void*)pH2DKey, TMR_FLAGS_NONE));
-                }
-                if (confComputeIsUvmKeyRotationPending(pGpu, pConfCompute))
-                {
-                    // If there is any UVM KR pending do not schedule the timer for timeout
-                    NV_ASSERT_OK_OR_RETURN(confComputeStopKeyRotationTimer(pGpu, pConfCompute, h2dKey));
-                }
-                else
-                {
-                    // no kernel KR pending, start/schedule timer for timeout
-                    NV_ASSERT_OK_OR_RETURN(tmrEventScheduleRelSec(pTmr,
-                                                                  pConfCompute->keyRotationTimeoutInfo[h2dIndex].pTimer,
-                                                                  pConfCompute->keyRotationTimeout));
-                }
+                NvU32 *pH2DKey = portMemAllocNonPaged(sizeof(NvU32));
+                *pH2DKey = h2dKey;
+                NV_ASSERT_OK_OR_RETURN(tmrEventCreate(pTmr, &pConfCompute->ppKeyRotationTimer[h2dIndex],
+                                                      keyRotationTimeoutCallback, (void*)pH2DKey, TMR_FLAGS_NONE));
             }
-            else
-            {
-                // if this is a kernel KR, check if there are any pending user KRs
-                NvU32 userH2DKey;
-                KEY_ROTATION_STATUS userKRStatus;
-                NvU32 keySpace;
-                for (keySpace = 0; keySpace < CC_KEYSPACE_SIZE; keySpace++)
-                {
-                    if (keySpace == CC_KEYSPACE_GSP)
-                        continue;
 
-                    confComputeGetKeyPairForKeySpace_HAL(pGpu, pConfCompute, keySpace, NV_FALSE, &userH2DKey, NULL);
-                    NV_ASSERT_OK_OR_RETURN(confComputeGetKeyRotationStatus(pConfCompute, userH2DKey, &userKRStatus));
-                    if (userKRStatus == KEY_ROTATION_STATUS_PENDING)
-                    {
-                        NV_PRINTF(LEVEL_INFO, "User key rotation pending on key 0x%x\n", userH2DKey);
-                        NV_ASSERT_OK_OR_RETURN(confComputeStopKeyRotationTimer(pGpu, pConfCompute, userH2DKey));
-                    }
-                }
+            // Schedule first callback. Timeouts are not enforced on kernel channels (UVM).
+            if (!confComputeGlobalKeyIsKernelPriv_HAL(pConfCompute, h2dKey))
+            {
+                NV_ASSERT_OK_OR_RETURN(tmrEventScheduleRelSec(pTmr,
+                                                              pConfCompute->ppKeyRotationTimer[h2dIndex],
+                                                              pConfCompute->keyRotationTimeout));
             }
 
             //
             // Notify clients of pending KR
-            // We can't schedule a workitem for notifying since it may get scheduled too late and
+            // We can't schedule a workitem for this since it may get scheduled too late and
             // we might have already crossed the upper threshold by then.
             //
-            {
-                CHANNEL_ITERATOR iter = {0};
-                KernelChannel *pKernelChannel = NULL;
-
-                // notify all channels
-                NV_ASSERT_OK_OR_RETURN(confComputeInitChannelIterForKey(pGpu, pConfCompute, h2dKey, &iter));
-                while(confComputeGetNextChannelForKey(pGpu, pConfCompute, &iter, h2dKey, &pKernelChannel) == NV_OK)
-                {
-                    NV_ASSERT_OK_OR_RETURN(kchannelUpdateNotifierMem(pKernelChannel, NV_CHANNELGPFIFO_NOTIFICATION_TYPE_KEY_ROTATION_STATUS,
-                                                                     pConfCompute->keyRotationCount[h2dIndex], 0, (NvU32)KEY_ROTATION_STATUS_PENDING));
-                    NV_PRINTF(LEVEL_INFO, "chid 0x%x has pending key rotation, writing notifier with val 0x%x\n",
-                    kchannelGetDebugTag(pKernelChannel), (NvU32)KEY_ROTATION_STATUS_PENDING);
-
-                    // send events to clients if registered
-                    kchannelNotifyEvent(pKernelChannel, NVC86F_NOTIFIERS_KEY_ROTATION, pConfCompute->keyRotationCount[h2dIndex],
-                                        KEY_ROTATION_STATUS_PENDING, NULL, 0);
-                }
-            }
+            NV_ASSERT_OK_OR_RETURN(notifyKeyRotationByKeyPair(pGpu, pConfCompute, h2dKey));
         }
 
         //
@@ -288,14 +260,12 @@ triggerKeyRotationByKeyPair
         // crossed but when encrypt stats are computed in the next callback lower threshold is crossed here.
         // In this case, schedule key rotation.
         //
+        CHANNEL_ITERATOR iter = {0};
+        KernelChannel *pKernelChannel = NULL;
+        NV_ASSERT_OK_OR_RETURN(confComputeInitChannelIterForKey(pGpu, pConfCompute, h2dKey, &iter));
+        if (confComputeGetNextChannelForKey(pGpu, pConfCompute, &iter, h2dKey, &pKernelChannel) != NV_OK)
         {
-            CHANNEL_ITERATOR iter = {0};
-            KernelChannel *pKernelChannel = NULL;
-            NV_ASSERT_OK_OR_RETURN(confComputeInitChannelIterForKey(pGpu, pConfCompute, h2dKey, &iter));
-            if (confComputeGetNextChannelForKey(pGpu, pConfCompute, &iter, h2dKey, &pKernelChannel) != NV_OK)
-            {
-                NV_ASSERT_OK_OR_RETURN(confComputePerformKeyRotation(pGpu, pConfCompute, h2dKey, d2hKey, NV_TRUE));
-            }
+            NV_ASSERT_OK_OR_RETURN(confComputePerformKeyRotation(pGpu, pConfCompute, h2dKey, d2hKey, NV_TRUE));
         }
     }
     return NV_OK;
@@ -311,11 +281,9 @@ keyRotationTimeoutCallback
 {
     ConfidentialCompute *pConfCompute = GPU_GET_CONF_COMPUTE(pGpu);
     NvU32 h2dKey, d2hKey;
-    NV_ASSERT_OR_RETURN(pEvent->pUserData != NULL, NV_ERR_INVALID_STATE);
     NvU32 key = *(NvU32*)pEvent->pUserData;
     confComputeGetKeyPairByKey(pConfCompute, key, &h2dKey, &d2hKey);
     NV_ASSERT_OK_OR_RETURN(confComputeSetKeyRotationStatus(pConfCompute, h2dKey, KEY_ROTATION_STATUS_FAILED_TIMEOUT));
-    NV_PRINTF(LEVEL_INFO, "Hit timeout on key 0x%x, triggering KR\n", h2dKey);
     return confComputePerformKeyRotation(pGpu, pConfCompute, h2dKey, d2hKey, NV_TRUE);
 }
 
@@ -399,12 +367,17 @@ isUpperThresholdCrossed
 )
 {
     NvU32 h2dIndex, d2hIndex;
-
-    NV_ASSERT_OK(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, h2dKey, &h2dIndex));
-    NV_ASSERT_OK(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, d2hKey, &d2hIndex));
-
-    return (confComputeIsUpperThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[h2dIndex]) ||
-            confComputeIsUpperThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[d2hIndex]));
+    NV_ASSERT_OK_OR_RETURN(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, h2dKey, &h2dIndex));
+    NV_ASSERT_OK_OR_RETURN(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, d2hKey, &d2hIndex));
+    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED))
+    {
+        return (pConfCompute->keyRotationCallbackCount[h2dIndex] > pConfCompute->upperThreshold.totalEncryptOps);
+    }
+    else
+    {
+        return (confComputeIsUpperThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[h2dIndex]) ||
+                confComputeIsUpperThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[d2hIndex]));
+    }
 }
 
 static NvBool
@@ -416,11 +389,17 @@ isLowerThresholdCrossed
 )
 {
     NvU32 h2dIndex, d2hIndex;
-    NV_ASSERT_OK(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, h2dKey, &h2dIndex));
-    NV_ASSERT_OK(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, d2hKey, &d2hIndex));
-
-    return (confComputeIsLowerThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[h2dIndex]) ||
-            confComputeIsLowerThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[d2hIndex]));
+    NV_ASSERT_OK_OR_RETURN(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, h2dKey, &h2dIndex));
+    NV_ASSERT_OK_OR_RETURN(confComputeGetKeySlotFromGlobalKeyId(pConfCompute, d2hKey, &d2hIndex));
+    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED))
+    {
+        return (pConfCompute->keyRotationCallbackCount[h2dIndex] > pConfCompute->lowerThreshold.totalEncryptOps);
+    }
+    else
+    {
+        return (confComputeIsLowerThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[h2dIndex]) ||
+                confComputeIsLowerThresholdCrossed(pConfCompute, &pConfCompute->aggregateStats[d2hIndex]));
+    }
 }
 
 static void
@@ -456,3 +435,160 @@ getKeyPairForKeySpace(NvU32 keySpace, NvBool bKernel, NvU32 *pGlobalH2DKey, NvU3
     *pGlobalH2DKey = CC_GKEYID_GEN(keySpace, localH2DKey);
     *pGlobalD2HKey = CC_GKEYID_GEN(keySpace, localD2HKey);
 }
+
+static NV_STATUS
+notifyKeyRotationByKeyPair
+(
+    OBJGPU *pGpu,
+    ConfidentialCompute *pConfCompute,
+    NvU32 h2dKey
+)
+{
+    KEY_ROTATION_STATUS status;
+    CHANNEL_ITERATOR iter = {0};
+    KernelChannel *pKernelChannel = NULL;
+    NV_ASSERT_OK_OR_RETURN(confComputeGetKeyRotationStatus(pConfCompute, h2dKey, &status));
+    //
+    // We expect this work item to be called soon after RM detects lower threshold is
+    // crossed and schedules this.
+    //
+    NV_ASSERT_OR_RETURN(status == KEY_ROTATION_STATUS_PENDING, NV_ERR_INVALID_STATE);
+
+	// notify all channels
+	NV_ASSERT_OK_OR_RETURN(confComputeInitChannelIterForKey(pGpu, pConfCompute, h2dKey, &iter));
+	while(confComputeGetNextChannelForKey(pGpu, pConfCompute, &iter, h2dKey, &pKernelChannel) == NV_OK)
+	{
+		NV_ASSERT_OK_OR_RETURN(kchannelUpdateNotifierMem(pKernelChannel, NV_CHANNELGPFIFO_NOTIFICATION_TYPE_KEY_ROTATION_STATUS,
+											             0, 0, (NvU32)status));
+		NV_PRINTF(LEVEL_INFO, "chid 0x%x has pending key rotation, writing notifier with val 0x%x\n",
+			kchannelGetDebugTag(pKernelChannel), (NvU32)status);
+
+		// send events to clients if registered
+		kchannelNotifyEvent(pKernelChannel, NVC86F_NOTIFIERS_KEY_ROTATION, 0, status, NULL, 0);
+	}
+    return NV_OK;
+}
+
+static void
+initKeyRotationRegistryOverrides
+(
+    OBJGPU *pGpu,
+    ConfidentialCompute *pConfCompute
+)
+{
+    if (pConfCompute->getProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_SUPPORTED))
+    {
+        NvU32 data;
+
+        if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION_THRESHOLD_DELTA, &data) == NV_OK)
+        {
+            if (data == 0)
+            {
+                NV_PRINTF(LEVEL_ERROR, "Illegal value for RmKeyRotationThresholdDelta.\n");
+                NV_PRINTF(LEVEL_ERROR, "Cancelling override of threshold delta.\n");
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_INFO, "Setting key rotation threshold delta to %u.\n", data);
+                pConfCompute->keyRotationLimitDelta = data;
+                NV_ASSERT_OK(confComputeSetKeyRotationThreshold(pConfCompute, pConfCompute->attackerAdvantage));
+            }
+        }
+        else if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION_LOWER_THRESHOLD, &data) == NV_OK)
+        {
+            const NvU32 lowerThreshold = data;
+
+            if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION_UPPER_THRESHOLD, &data) == NV_OK)
+            {
+                const NvU32 upperThreshold = data;
+
+                if (upperThreshold > lowerThreshold)
+                {
+                    NV_PRINTF(LEVEL_INFO, "Setting key rotation lower threshold to %u and upper threshold to %u.\n",
+                              lowerThreshold, upperThreshold);
+                    pConfCompute->keyRotationUpperLimit = upperThreshold;
+                    pConfCompute->keyRotationLowerLimit = lowerThreshold;
+                }
+                else
+                {
+                    NV_PRINTF(LEVEL_ERROR, "RmKeyRotationUpperThreshold must be greater than RmKeyRotationLowerThreshold.\n");
+                    NV_PRINTF(LEVEL_ERROR, "Cancelling override of upper and lower key rotation thresholds.\n");
+                }
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_ERROR, "RmKeyRotationUpperThreshold must be set if RmKeyRotationLowerThreshold is set.\n");
+                NV_PRINTF(LEVEL_ERROR, "Cancelling override of upper and lower key rotation thresholds.\n");
+            }
+        }
+
+        if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION_TIMEOUT_IN_SEC, &data) == NV_OK)
+        {
+            const NvU32 minTimeout = 2;
+
+            if (data >= minTimeout)
+            {
+                NV_PRINTF(LEVEL_INFO, "Setting key rotation user-mode timeout to %u seconds.\n", data);
+                pConfCompute->keyRotationTimeout = data;
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_ERROR, "Key rotation user-mode timeout must be greater than or equal to %u.\n", minTimeout);
+                NV_PRINTF(LEVEL_ERROR, "Cancelling override of user-mode timeout.\n");
+            }
+        }
+
+        if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_KEY_ROTATION, &data) == NV_OK)
+        {
+            if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_KEY_ROTATION, _ENABLED, _YES, data))
+            {
+                NV_PRINTF(LEVEL_INFO, "Confidential Compute key rotation enabled via regkey override.\n");
+                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_ENABLED, NV_TRUE);
+                pConfCompute->keyRotationEnableMask = data;
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_INFO, "Confidential Compute key rotation disabled via regkey override.\n");
+                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_ENABLED, NV_FALSE);
+                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_SUPPORTED, NV_FALSE);
+            }
+        }
+        else if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_DUMMY_KEY_ROTATION, &data) == NV_OK)
+        {
+            if (FLD_TEST_DRF(_REG_STR, _RM_CONF_COMPUTE_DUMMY_KEY_ROTATION, _ENABLED, _YES, data))
+            {
+                NV_PRINTF(LEVEL_INFO, "Confidential Compute dummy key rotation enabled via regkey override.\n");
+                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED, NV_TRUE);
+                pConfCompute->keyRotationEnableMask = data;
+
+                pConfCompute->lowerThreshold.totalBytesEncrypted = NV_U64_MAX;
+                pConfCompute->lowerThreshold.totalEncryptOps = KEY_ROTATION_LOWER_THRESHOLD;
+                pConfCompute->upperThreshold.totalBytesEncrypted = NV_U64_MAX;
+                pConfCompute->upperThreshold.totalEncryptOps = KEY_ROTATION_UPPER_THRESHOLD;
+
+                if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_DUMMY_KEY_ROTATION_LOWER_THRESHOLD, &data) == NV_OK)
+                {
+                    pConfCompute->lowerThreshold.totalEncryptOps = data;
+                }
+
+                if (osReadRegistryDword(pGpu, NV_REG_STR_RM_CONF_COMPUTE_DUMMY_KEY_ROTATION_UPPER_THRESHOLD, &data) == NV_OK)
+                {
+                    pConfCompute->upperThreshold.totalEncryptOps = data;
+                }
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_INFO, "Confidential Compute dummy key rotation disabled via regkey override.\n");
+                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED, NV_FALSE);
+                pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_SUPPORTED, NV_FALSE);
+            }
+        }
+        else
+        {
+            NV_PRINTF(LEVEL_INFO, "Confidential Compute key rotation (dummy and production) are disabled.\n");
+            pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_ENABLED, NV_FALSE);
+            pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_DUMMY_KEY_ROTATION_ENABLED, NV_FALSE);
+            pConfCompute->setProperty(pConfCompute, PDB_PROP_CONFCOMPUTE_KEY_ROTATION_SUPPORTED, NV_FALSE);
+        }
+    }
+}
\ No newline at end of file
