@@ -57,7 +57,6 @@ static struct
 } stuckIntr[MC_ENGINE_IDX_MAX];
 
 static NvBool _intrServiceStallExactList(OBJGPU *pGpu, Intr *pIntr, MC_ENGINE_BITVECTOR *pEngines);
-static void _intrLogLongRunningInterrupts(Intr *pIntr);
 static void _intrInitServiceTable(OBJGPU *pGpu, Intr *pIntr);
 
 
@@ -144,8 +143,6 @@ intrServiceStall_IMPL(OBJGPU *pGpu, Intr *pIntr)
         intrProcessDPCQueue_HAL(pGpu, pIntr);
     }
 
-    _intrLogLongRunningInterrupts(pIntr);
-
 exit:
     return;
 }
@@ -173,7 +170,7 @@ subdeviceCtrlCmdMcServiceInterrupts_IMPL
 
         //
         // Force kernel-RM to service interrupts from GSP-RM. This will allow
-        // kernel-RM to write notifiers and send an ack back to GSP. 
+        // kernel-RM to write notifiers and send an ack back to GSP.
         // GSP waits for this ack before clearing fast path POSSIBLE_ERR interrupt.
         //
         if (pGpu->getProperty(pGpu, PDB_PROP_GPU_FASTPATH_SEQ_ENABLED))
@@ -595,13 +592,24 @@ intrGetVectorFromEngineId_IMPL
     NvBool  bNonStall
 )
 {
+    NvU32 intrVector;
     INTR_TABLE_ENTRY *pEntry = intrGetInterruptTableEntryFromEngineId(pGpu,
         pIntr,
         mcEngineId,
         bNonStall);
 
     NV_ASSERT_OR_RETURN(pEntry != NULL, NV_INTR_VECTOR_INVALID);
-    return bNonStall ? pEntry->intrVectorNonStall : pEntry->intrVector;
+    intrVector = bNonStall ? pEntry->intrVectorNonStall : pEntry->intrVector;
+
+    if (intrVector == NV_INTR_VECTOR_INVALID)
+    {
+        NV_PRINTF(LEVEL_ERROR, "mcEngineIdx %d with bNonStall = %d has invalid vector\n",
+                                mcEngineId, bNonStall);
+
+        NV_ASSERT_OR_RETURN(intrVector != NV_INTR_VECTOR_INVALID, NV_INTR_VECTOR_INVALID);
+    }
+
+    return intrVector;
 }
 
 
@@ -792,6 +800,29 @@ intrDestruct_IMPL
 }
 
 
+NV_STATUS
+intrStatePreInitLocked_IMPL
+(
+    OBJGPU *pGpu,
+    Intr   *pIntr
+)
+{
+    NV_STATUS status = NV_OK;
+
+    for (NvU32 i = 0; i < NV_ARRAY_ELEMENTS(pIntr->subtreeMap); i++)
+    {
+        pIntr->subtreeMap[i].subtreeStart = NV2080_INTR_INVALID_SUBTREE;
+        pIntr->subtreeMap[i].subtreeEnd   = NV2080_INTR_INVALID_SUBTREE;
+    }
+
+    NV_ASSERT_OK_OR_GOTO(status, intrInitInterruptTable_HAL(pGpu, pIntr), exit);
+    _intrInitServiceTable(pGpu, pIntr);
+
+exit:
+
+    return status;
+}
+
 NV_STATUS
 intrStateInitUnlocked_IMPL
 (
@@ -800,7 +831,6 @@ intrStateInitUnlocked_IMPL
 )
 {
     NvU32 data = 0;
-    NvU32 i;
 
     if (osReadRegistryDword(pGpu,
                             NV_REG_STR_RM_INTR_DETAILED_LOGS, &data) == NV_OK)
@@ -815,12 +845,6 @@ intrStateInitUnlocked_IMPL
 
     _intrInitRegistryOverrides(pGpu, pIntr);
 
-    for (i = 0; i < NV_ARRAY_ELEMENTS(pIntr->subtreeMap); ++i)
-    {
-        pIntr->subtreeMap[i].subtreeStart = NV2080_INTR_INVALID_SUBTREE;
-        pIntr->subtreeMap[i].subtreeEnd   = NV2080_INTR_INVALID_SUBTREE;
-    }
-
     return NV_OK;
 }
 
@@ -832,7 +856,6 @@ intrStateInitLocked_IMPL
     Intr     *pIntr
 )
 {
-    NV_STATUS status = NV_OK;
     // Enable interrupts in the HAL
     pIntr->halIntrEnabled = NV_TRUE;
 
@@ -850,12 +873,8 @@ intrStateInitLocked_IMPL
         pKernelGmmu->uvmSharedIntrRmOwnsMask = RM_UVM_SHARED_INTR_MASK_ALL;
     }
 
-    NV_ASSERT_OK_OR_GOTO(status, intrInitInterruptTable_HAL(pGpu, pIntr), exit);
-    _intrInitServiceTable(pGpu, pIntr);
-
     if (pIntr->getProperty(pIntr, PDB_PROP_INTR_USE_INTR_MASK_FOR_LOCKING))
     {
-        intrGetIntrMask_HAL(pGpu, pIntr, &pIntr->intrMask.engMaskOrig, NULL /* threadstate */);
         if (RMCFG_FEATURE_PLATFORM_WINDOWS)
         {
             MC_ENGINE_BITVECTOR engines;
@@ -869,9 +888,7 @@ intrStateInitLocked_IMPL
         // Hypervisor will set the intr unblocked mask later at the time of SWRL init.
     }
 
-exit:
-
-    return status;
+    return NV_OK;
 }
 
 void
@@ -957,8 +974,7 @@ _intrInitRegistryOverrides
     if (RMCFG_FEATURE_PLATFORM_WINDOWS || hypervisorIsVgxHyper())
     {
         // Enable IntrMask Locking by default if supported
-        if (pIntr->getProperty(pIntr, PDB_PROP_INTR_MASK_SUPPORTED) &&
-            (!IS_VIRTUAL(pGpu) && !IS_GSP_CLIENT(pGpu)))
+        if (pIntr->getProperty(pIntr, PDB_PROP_INTR_MASK_SUPPORTED) && !IS_VIRTUAL(pGpu))
         {
             pIntr->setProperty(pIntr, PDB_PROP_INTR_USE_INTR_MASK_FOR_LOCKING, NV_TRUE);
         }
@@ -1075,18 +1091,14 @@ _intrInitServiceTable
     Intr *pIntr
 )
 {
-    ENGSTATE_ITER iter = gpuGetEngstateIter(pGpu);
-    OBJENGSTATE *pEngstate;
+    GPU_CHILD_ITER iter = {0};
+    IntrService *pIntrService;
 
     portMemSet(pIntr->intrServiceTable, 0, sizeof(pIntr->intrServiceTable));
 
-    while (gpuGetNextEngstate(pGpu, &iter, &pEngstate))
+    while ((pIntrService = GPU_GET_NEXT_CHILD_OF_TYPE(pGpu, &iter, IntrService)))
     {
-        IntrService *pIntrService = dynamicCast(pEngstate, IntrService);
-        if (pIntrService != NULL)
-        {
-            intrservRegisterIntrService(pGpu, pIntrService, pIntr->intrServiceTable);
-        }
+        intrservRegisterIntrService(pGpu, pIntrService, pIntr->intrServiceTable);
     }
 
     gpuRegisterGenericKernelFalconIntrService(pGpu, pIntr->intrServiceTable);
@@ -1103,7 +1115,6 @@ NvU32 intrServiceInterruptRecords_IMPL
     IntrService *pIntrService = pIntr->intrServiceTable[engineIdx].pInterruptService;
     NvU32 ret = 0;
     NvBool bShouldService;
-    NvU64 intrTiming, intrTiming2;
     IntrServiceClearInterruptArguments clearParams = {engineIdx};
     IntrServiceServiceInterruptArguments serviceParams = {engineIdx};
 
@@ -1125,21 +1136,9 @@ NvU32 intrServiceInterruptRecords_IMPL
 
     if (bShouldService)
     {
-        osGetPerformanceCounter(&intrTiming);
-
-        GSP_TRACE_RATS_ADD_RECORD(NV_RATS_GSP_TRACE_TYPE_INTR_INFO, pGpu, (NvU32) engineIdx);
         GSP_TRACE_RATS_ADD_RECORD(NV_RATS_GSP_TRACE_TYPE_INTR_START, pGpu, 0);
         ret = intrservServiceInterrupt(pGpu, pIntrService, &serviceParams);
-        GSP_TRACE_RATS_ADD_RECORD(NV_RATS_GSP_TRACE_TYPE_INTR_END, pGpu, 0);
-
-        osGetPerformanceCounter(&intrTiming2);
-        intrTiming = intrTiming2 - intrTiming;
-        if (intrTiming > LONG_INTR_LOG_LENGTH_NS)
-        {
-            pIntr->longIntrStats[engineIdx].intrCount++;
-            if (intrTiming > pIntr->longIntrStats[engineIdx].intrLength)
-                pIntr->longIntrStats[engineIdx].intrLength = intrTiming;
-        }
+        GSP_TRACE_RATS_ADD_RECORD(NV_RATS_GSP_TRACE_TYPE_INTR_END, pGpu, (NvU32) engineIdx);
     }
     return ret;
 }
@@ -1464,29 +1463,6 @@ _intrExitCriticalSection
     }
 }
 
-static void
-_intrLogLongRunningInterrupts(Intr *pIntr)
-{
-    NvU64 now;
-    osGetPerformanceCounter(&now);
-
-    for (NvU32 i = 0; i < MC_ENGINE_IDX_MAX; ++i)
-    {
-        if (pIntr->longIntrStats[i].intrCount > 0)
-        {
-            if (now - pIntr->longIntrStats[i].lastPrintTime > LONG_INTR_LOG_RATELIMIT_NS)
-            {
-                NV_PRINTF(LEVEL_WARNING, "%u long-running interrupts (%llu ns or slower) from engine %u, longest taking %llu ns\n",
-                          pIntr->longIntrStats[i].intrCount, LONG_INTR_LOG_LENGTH_NS, i, pIntr->longIntrStats[i].intrLength);
-
-                pIntr->longIntrStats[i].intrCount = 0;
-                pIntr->longIntrStats[i].intrLength = 0;
-                pIntr->longIntrStats[i].lastPrintTime = now;
-            }
-        }
-    }
-}
-
 static NvBool
 _intrServiceStallExactList
 (
@@ -1669,9 +1645,6 @@ done:
     // allow the isr to come in.
     _intrExitCriticalSection(pGpu, pIntr, &intrMaskCtx);
 
-    // Delay prints until after exiting critical sections to save perf impact
-    _intrLogLongRunningInterrupts(pIntr);
-
     NV_ASSERT_OK(resservRestoreTlsCallContext(pOldContext));
 }
 
@@ -1818,3 +1791,78 @@ NV_STATUS intrSetInterruptEntry_IMPL
     pIntr->vectorToMcIdx[tree][vector] = *pEntry;
     return NV_OK;
 }
+
+
+NvU64
+intrGetUvmSharedLeafEnDisableMask_IMPL
+(
+    OBJGPU *pGpu,
+    Intr   *pIntr
+)
+{
+    NvU64 mask = 0;
+    NV2080_INTR_CATEGORY_SUBTREE_MAP uvmShared;
+
+    // GSP RM services both MMU non-replayable fault and FIFO interrupts
+    if (IS_GSP_CLIENT(pGpu))
+    {
+        return ~mask;
+    }
+
+    NvU32 locklessRmVectors[2];
+    intrGetLocklessVectorsInRmSubtree_HAL(pGpu, pIntr, &locklessRmVectors);
+
+    NV_ASSERT_OK(intrGetSubtreeRange(pIntr,
+                                     NV2080_INTR_CATEGORY_UVM_SHARED,
+                                     &uvmShared));
+    //
+    // Ascertain that we only have 1 client subtree (we assume
+    // this since we cache only 64 bits).
+    //
+    NV_ASSERT(uvmShared.subtreeStart == uvmShared.subtreeEnd);
+
+    //
+    // Ascertain that we only have 2 subtrees as this is what we currently
+    // support by only caching 64 bits
+    //
+    NV_ASSERT(
+        (NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_END(uvmShared.subtreeEnd) - 1) ==
+        NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_START(uvmShared.subtreeStart));
+
+    NvU32 i;
+    for (i = 0; i < NV_ARRAY_ELEMENTS(locklessRmVectors); ++i)
+    {
+        NvU32 vector = locklessRmVectors[i];
+        if (vector == NV_INTR_VECTOR_INVALID)
+        {
+            continue;
+        }
+
+        // Ascertain that they're in the first leaf
+        NV_ASSERT(
+            NV_CTRL_INTR_GPU_VECTOR_TO_LEAF_REG(vector) ==
+            NV_CTRL_INTR_SUBTREE_TO_LEAF_IDX_START(uvmShared.subtreeStart));
+
+        mask |= NVBIT64(NV_CTRL_INTR_GPU_VECTOR_TO_LEAF_BIT(vector));
+    }
+
+    mask <<= 32;
+
+    return ~mask;
+}
+
+NV_STATUS
+intrRefetchInterruptTable_IMPL
+(
+    OBJGPU *pGpu,
+    Intr   *pIntr
+)
+{
+    NV_ASSERT_OK_OR_RETURN(intrStateUnload_HAL(pGpu, pIntr, GPU_STATE_FLAGS_PRESERVING));
+
+    NV_ASSERT_OK_OR_RETURN(intrInitInterruptTable_HAL(pGpu, pIntr));
+
+    NV_ASSERT_OK_OR_RETURN(intrStateLoad_HAL(pGpu, pIntr, GPU_STATE_FLAGS_PRESERVING));
+
+    return NV_OK;
+}
