@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2020-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2020-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -36,6 +36,8 @@
 
 #include "gpu/bus/kern_bus.h"
 
+#include "gpu/mem_mgr/ce_utils.h"
+
 #include "kernel/gpu/conf_compute/ccsl.h"
 
 #include "class/cl0005.h"      // NV01_EVENT
@@ -56,37 +58,79 @@ memmgrGetMemTransferType
 (
     MemoryManager    *pMemoryManager,
     TRANSFER_SURFACE *pDst,
-    TRANSFER_SURFACE *pSrc
+    TRANSFER_SURFACE *pSrc,
+    NvU32             flags
 )
 {
     TRANSFER_TYPE transferType        = TRANSFER_TYPE_PROCESSOR;
-    OBJGPU    *pGpu       = ENG_GET_GPU(pMemoryManager);
-    KernelBus *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
+    OBJGPU       *pGpu                = ENG_GET_GPU(pMemoryManager);
+    KernelBus    *pKernelBus          = GPU_GET_KERNEL_BUS(pGpu);
 
-    //
-    // In case of copy, both dest and src will be passed
-    // In case of memset/memread/memwrite either dest or src will be passed
-    //
-    if ((pDst != NULL) && (pSrc != NULL) &&
-        (memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM) &&
-        (memdescGetAddressSpace(pSrc->pMemDesc) == ADDR_SYSMEM))
+    if ((pDst == NULL || memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM) &&
+        (pSrc == NULL || memdescGetAddressSpace(pSrc->pMemDesc) == ADDR_SYSMEM))
     {
+        //
+        // If the operation only touches sysmem, use processor copy
+        //
         transferType = TRANSFER_TYPE_PROCESSOR;
     }
-    else if (((pDst != NULL) &&
-             (memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM)) ||
-             ((pSrc != NULL) &&
-             (memdescGetAddressSpace(pSrc->pMemDesc) == ADDR_SYSMEM)))
+    else if (flags & TRANSFER_FLAGS_PREFER_CE)
     {
-        transferType = TRANSFER_TYPE_PROCESSOR;
+        if (IS_SIMULATION(pGpu) && pSrc != NULL)
+        {
+            //
+            // This is significantly faster on fmodel for S/R (5min vs. 5sec) because of the
+            // backdoor memory reads and writes.
+            // Memset is not currently supported
+            //
+            return TRANSFER_TYPE_BAR0;
+        }
+
+        CeUtils *pCeUtils = pMemoryManager->pCeUtils;
+
+        if (pCeUtils != NULL)
+        {
+            NvBool bDstCompr = pDst != NULL && memmgrIsKind_HAL(pMemoryManager, FB_IS_KIND_COMPRESSIBLE,
+                                                                memdescGetPteKind(pDst->pMemDesc));
+            NvBool bSrcCompr = pSrc != NULL && memmgrIsKind_HAL(pMemoryManager, FB_IS_KIND_COMPRESSIBLE,
+                                                                memdescGetPteKind(pSrc->pMemDesc));
+            if (!(bSrcCompr || bDstCompr) || pCeUtils->bUseVasForCeCopy)
+            {
+                return TRANSFER_TYPE_CE;
+            }
+        }
+
+        // On Emulation we may lack CE support so preventing excessive debug spew.
+        if (!IS_EMULATION(pGpu))
+        {
+            NV_PRINTF(LEVEL_WARNING, "Can't copy using CE, falling back to other methods\n");
+        }
     }
-    else if (kbusIsBarAccessBlocked(pKernelBus))
+    else if (kbusIsBarAccessBlocked(pKernelBus) &&
+            (!gpuIsCCDevToolsModeEnabled(pGpu) || !(flags & TRANSFER_FLAGS_PREFER_PROCESSOR)))
     {
         transferType = TRANSFER_TYPE_GSP_DMA;
     }
+
     return transferType;
 }
 
+static NV_STATUS
+memmgrCheckSurfaceBounds
+(
+    TRANSFER_SURFACE *pSurface,
+    NvU64             size
+)
+{
+    NV_ASSERT_OR_RETURN(pSurface != NULL, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(pSurface->pMemDesc != NULL, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(size != 0, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(pSurface->offset <= pSurface->pMemDesc->Size, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OR_RETURN(pSurface->offset + size <= pSurface->pMemDesc->Size, NV_ERR_INVALID_ARGUMENT);
+
+    return NV_OK;
+}
+
 static NV_STATUS
 _memmgrAllocAndMapSurface
 (
@@ -108,9 +152,11 @@ _memmgrAllocAndMapSurface
 
     NV_ASSERT_OK_OR_RETURN(
         memdescCreate(ppMemDesc, pGpu, size, RM_PAGE_SIZE, NV_TRUE,
-                      ADDR_SYSMEM, NV_MEMORY_UNCACHED, flags));
+                      ADDR_SYSMEM, NV_MEMORY_CACHED, flags));
 
-    NV_ASSERT_OK_OR_GOTO(status, memdescAlloc(*ppMemDesc), failed);
+    memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_77, 
+                    (*ppMemDesc));
+    NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR, status, failed);
 
     NV_ASSERT_OK_OR_GOTO(status,
         memdescMapOld(*ppMemDesc, 0, size, NV_TRUE, NV_PROTECT_READ_WRITE,
@@ -172,19 +218,13 @@ _memmgrMemReadOrWriteWithGsp
     void *pStagingBufPriv = NULL;
     RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
     ConfidentialCompute *pConfCompute = GPU_GET_CONF_COMPUTE(pGpu);
-    if (gpuIsCCFeatureEnabled(pGpu))
-    {
-        NV_ASSERT_OR_RETURN(pConfCompute->getProperty(pCC,
-                                    PDB_PROP_CONFCOMPUTE_ENCRYPT_ENABLED),
-                            NV_ERR_INVALID_STATE);
-    }
 
     // Do not expect GSP to be used for reading/writing from/to sysmem
     if (memdescGetAddressSpace(pDst->pMemDesc) == ADDR_SYSMEM)
         return NV_ERR_NOT_SUPPORTED;
 
     // Allocate and map the staging buffer
-    NV_ASSERT_OK_OR_RETURN(
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
         _memmgrAllocAndMapSurface(pGpu, size, &pStagingBuf, &pStagingBufMap,
                                   &pStagingBufPriv));
 
@@ -196,10 +236,18 @@ _memmgrMemReadOrWriteWithGsp
     {
         if (gpuIsCCFeatureEnabled(pGpu))
         {
-            NV_ASSERT_OK_OR_GOTO(status,
-                ccslEncrypt_HAL(pConfCompute->pDmaCcslCtx, size, pBuf, NULL, 0,
-                                pStagingBufMap, gspParams.authTag),
-                failed);
+            status = ccslEncrypt_HAL(pConfCompute->pDmaCcslCtx, size, pBuf, NULL, 0,
+                                     pStagingBufMap, gspParams.authTag);
+            if (status != NV_OK)
+            {
+                if (status == NV_ERR_INSUFFICIENT_RESOURCES)
+                {
+                    // We hit potential IV overflow, this is fatal.
+                    NV_PRINTF(LEVEL_ERROR, "Fatal error detected in GSP-DMA encrypt: IV Overflow!\n");
+                    confComputeSetErrorState(pGpu, pConfCompute);
+                }
+                goto failed;
+            }
         }
         else
         {
@@ -258,10 +306,15 @@ _memmgrMemReadOrWriteWithGsp
     {
         if (gpuIsCCFeatureEnabled(pGpu))
         {
-            NV_ASSERT_OK_OR_GOTO(status,
-                ccslDecrypt_HAL(pConfCompute->pDmaCcslCtx, size, pStagingBufMap,
-                                NULL, NULL, 0, pBuf, gspParams.authTag),
-                failed);
+            status = ccslDecrypt_HAL(pConfCompute->pDmaCcslCtx, size, pStagingBufMap,
+                                     NULL, NULL, 0, pBuf, gspParams.authTag);
+            if (status != NV_OK)
+            {
+                // Failure in GSP-DMA decrypt is considered fatal.
+                NV_PRINTF(LEVEL_ERROR, "Fatal error detected in GSP-DMA decrypt: 0x%x!\n", status);
+                confComputeSetErrorState(pGpu, pConfCompute);
+                goto failed;
+            }
         }
         else
         {
@@ -299,6 +352,7 @@ _memmgrMemcpyWithGsp
     NvU8 *pMap = NULL;
     void *pPriv = NULL;
     RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
+    ConfidentialCompute *pConfCompute = GPU_GET_CONF_COMPUTE(pGpu);
 
     //
     // Do not expect GSP to be used for copying data b/w two surfaces
@@ -313,7 +367,7 @@ _memmgrMemcpyWithGsp
     }
 
     // Allocate and map the bounce buffer
-    NV_ASSERT_OK_OR_RETURN(
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
         _memmgrAllocAndMapSurface(pGpu, size, &pStagingBuf, &pStagingBufMap,
                                   &pStagingBufPriv));
 
@@ -330,10 +384,29 @@ _memmgrMemcpyWithGsp
                           NV_PROTECT_READ_WRITE, (void**)&pMap, &pPriv),
             failed);
 
-        // Copy to staging buffer
-        portMemCopy(pStagingBufMap, size, pMap + pSrc->offset, size);
+        // Copy to staging buffer, encrypting first if CC mode
+        if (gpuIsCCFeatureEnabled(pGpu))
+        {
+            status = ccslEncrypt_HAL(pConfCompute->pDmaCcslCtx, size, pMap + pSrc->offset,
+                                NULL, 0, pStagingBufMap, gspParams.authTag);
+            if (status == NV_ERR_INSUFFICIENT_RESOURCES)
+            {
+                //
+                // We hit potential IV overflow, this is fatal.
+                // Status is checked below, make sure we unmap memory first.
+                //
+                NV_PRINTF(LEVEL_ERROR, "Fatal error detected in GSP-DMA encrypt: IV Overflow!\n");
+                confComputeSetErrorState(pGpu, pConfCompute);
+            }
+        }
+        else
+        {
+            portMemCopy(pStagingBufMap, size, pMap + pSrc->offset, size);
+        }
 
+        // Be sure to unmap memory before potentially taking cleanup path
         memdescUnmapOld(pSrc->pMemDesc, NV_TRUE, 0, (void*)pMap, pPriv);
+        NV_ASSERT_OK_OR_GOTO(status, status, failed);
 
         // Source surface in unprotected sysmem
         gspParams.src.baseAddr = memdescGetPhysAddr(pStagingBuf, AT_GPU, 0);
@@ -396,9 +469,29 @@ _memmgrMemcpyWithGsp
                           NV_PROTECT_READ_WRITE, (void**)&pMap, &pPriv),
             failed);
 
-        portMemCopy(pMap + pDst->offset, size, pStagingBufMap, size);
+        if (gpuIsCCFeatureEnabled(pGpu))
+        {
+            status = ccslDecrypt_HAL(pConfCompute->pDmaCcslCtx, size, pStagingBufMap,
+                                NULL, NULL, 0, pMap + pDst->offset, gspParams.authTag);
+            if (status != NV_OK)
+            {
+                //
+                // Failure in GSP-DMA decrypt is considered fatal.
+                // Just print and set fatal state here, status is checked below after
+                // unmappin memory.
+                //
+                NV_PRINTF(LEVEL_ERROR, "Fatal error detected in GSP-DMA decrypt: 0x%x!\n", status);
+                confComputeSetErrorState(pGpu, pConfCompute);
+            }
+        }
+        else
+        {
+            portMemCopy(pMap + pDst->offset, size, pStagingBufMap, size);
+        }
 
+        // Be sure to unmap memory before potentially taking cleanup path
         memdescUnmapOld(pDst->pMemDesc, NV_TRUE, 0, (void*)pMap, pPriv);
+        NV_ASSERT_OK_OR_GOTO(status, status, failed);
     }
 
 failed:
@@ -468,33 +561,30 @@ memmgrMemCopyWithTransferType
 )
 {
     OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
-    NvU8 *pSrc;
-    NvU8 *pDst;
 
     // Sanitize the input
-    NV_ASSERT_OR_RETURN(pDstInfo != NULL, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pSrcInfo != NULL, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pDstInfo->pMemDesc != NULL, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pSrcInfo->pMemDesc != NULL, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OK_OR_RETURN(memmgrCheckSurfaceBounds(pDstInfo, size));
+    NV_ASSERT_OK_OR_RETURN(memmgrCheckSurfaceBounds(pSrcInfo, size));
     NV_ASSERT_OR_RETURN(!memdescDescIsEqual(pDstInfo->pMemDesc, pSrcInfo->pMemDesc),
                         NV_ERR_INVALID_ARGUMENT);
 
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
-            pDst = memdescMapInternal(pGpu, pDstInfo->pMemDesc, TRANSFER_FLAGS_NONE);
-            NV_ASSERT_OR_RETURN(pDst != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
-            pSrc = memdescMapInternal(pGpu, pSrcInfo->pMemDesc, TRANSFER_FLAGS_NONE);
-            if (pSrc == NULL)
             {
-                memdescUnmapInternal(pGpu, pDstInfo->pMemDesc, 0);
-                NV_ASSERT_OR_RETURN(0, NV_ERR_INSUFFICIENT_RESOURCES);
+                NvU8 *pDst = memmgrMemBeginTransfer(pMemoryManager, pDstInfo, size, flags);
+                NvU8 *pSrc = memmgrMemBeginTransfer(pMemoryManager, pSrcInfo, size, flags);
+
+                if (pDst != NULL && pSrc != NULL)
+                {
+                    portMemCopy(pDst, size, pSrc, size);
+                }
+
+                memmgrMemEndTransfer(pMemoryManager, pSrcInfo, size, flags);
+                memmgrMemEndTransfer(pMemoryManager, pDstInfo, size, flags);
+
+                NV_CHECK_OR_RETURN(LEVEL_ERROR, pDst != NULL && pSrc != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
             }
-
-            portMemCopy(pDst + pDstInfo->offset, size, pSrc + pSrcInfo->offset, size);
-
-            memdescUnmapInternal(pGpu, pSrcInfo->pMemDesc, TRANSFER_FLAGS_NONE);
-            memdescUnmapInternal(pGpu, pDstInfo->pMemDesc, flags);
             break;
         case TRANSFER_TYPE_GSP_DMA:
             if (IS_GSP_CLIENT(pGpu))
@@ -508,13 +598,73 @@ memmgrMemCopyWithTransferType
             }
             break;
         case TRANSFER_TYPE_CE:
-            NV_PRINTF(LEVEL_INFO, "Add call to CE\n");
+            {
+                NV_ASSERT_OR_RETURN(pMemoryManager->pCeUtils != NULL, NV_ERR_INVALID_STATE);
+
+                CEUTILS_MEMCOPY_PARAMS  params = {0};
+
+                params.pDstMemDesc = pDstInfo->pMemDesc;
+                params.dstOffset   = pDstInfo->offset;
+                params.pSrcMemDesc = pSrcInfo->pMemDesc;
+                params.srcOffset   = pSrcInfo->offset;
+                params.length      = size;
+
+                return ceutilsMemcopy(pMemoryManager->pCeUtils, &params);
+            }
+            break;
+        case TRANSFER_TYPE_CE_PRI:
+            NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_STATE);
+            break;
+        case TRANSFER_TYPE_BAR0:
+            NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_STATE);
             break;
     }
 
     return NV_OK;
 }
 
+static NV_STATUS
+_memmgrMemReadOrWriteUsingStagingBuffer
+(
+    MemoryManager    *pMemoryManager,
+    TRANSFER_SURFACE *pAlloc,
+    void             *pBuf,
+    NvU64             size,
+    NvU32             transferType,
+    NvBool            bRead
+)
+{
+    MEMORY_DESCRIPTOR *pStagingBuf = NULL;
+    void *pStagingBufMap = NULL;
+    void *pStagingBufPriv = NULL;
+    TRANSFER_SURFACE staging = {0};
+    TRANSFER_SURFACE *pSrc = bRead ? pAlloc : &staging;
+    TRANSFER_SURFACE *pDst = bRead ? &staging : pAlloc;
+    NV_STATUS status = NV_OK;
+
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+        _memmgrAllocAndMapSurface(ENG_GET_GPU(pMemoryManager), size, &pStagingBuf, &pStagingBufMap,
+                                  &pStagingBufPriv));
+    staging.pMemDesc = pStagingBuf;
+
+    if (!bRead)
+    {
+        portMemCopy(pStagingBufMap, size, pBuf, size);
+    }
+
+    NV_ASSERT_OK_OR_GOTO(status, memmgrMemCopyWithTransferType(pMemoryManager, pDst, pSrc, size, transferType, 0), failed);
+
+    if (bRead)
+    {
+        portMemCopy(pBuf, size, pStagingBufMap, size);
+    }
+
+failed:
+    _memmgrUnmapAndFreeSurface(pStagingBuf, pStagingBufMap, pStagingBufPriv);
+
+    return status;
+}
+
 /*!
  * @brief This function is used for setting a memory region to a constant state
  *        using a specified memory transfer technique
@@ -537,23 +687,19 @@ memmgrMemSetWithTransferType
 )
 {
     OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
-    NvU8 *pDst;
 
     // Sanitize the input
-    NV_ASSERT_OR_RETURN(pDstInfo != NULL, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pDstInfo->pMemDesc != NULL, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(size > 0, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pDstInfo->offset + size <= pDstInfo->pMemDesc->Size, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OK_OR_RETURN(memmgrCheckSurfaceBounds(pDstInfo, size));
 
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
-            pDst = memdescMapInternal(pGpu, pDstInfo->pMemDesc, TRANSFER_FLAGS_NONE);
-            NV_ASSERT_OR_RETURN(pDst != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
-
-            portMemSet(pDst + pDstInfo->offset, value, size);
-
-            memdescUnmapInternal(pGpu, pDstInfo->pMemDesc, flags);
+            {
+                NvU8 *pDst = memmgrMemBeginTransfer(pMemoryManager, pDstInfo, size, flags);
+                NV_CHECK_OR_RETURN(LEVEL_ERROR, pDst != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
+                portMemSet(pDst, value, size);
+                memmgrMemEndTransfer(pMemoryManager, pDstInfo, size, flags);
+            }
             break;
         case TRANSFER_TYPE_GSP_DMA:
             if (IS_GSP_CLIENT(pGpu))
@@ -567,96 +713,32 @@ memmgrMemSetWithTransferType
             }
             break;
         case TRANSFER_TYPE_CE:
-            NV_PRINTF(LEVEL_INFO, "Add call to CE\n");
+            {
+                NV_ASSERT_OR_RETURN(pMemoryManager->pCeUtils != NULL, NV_ERR_INVALID_STATE);
+
+                CEUTILS_MEMSET_PARAMS  params = {0};
+
+                params.pMemDesc = pDstInfo->pMemDesc;
+                params.offset   = pDstInfo->offset;
+                params.length   = size;
+                params.pattern  = value;
+
+                return ceutilsMemset(pMemoryManager->pCeUtils, &params);
+            }
             break;
+        case TRANSFER_TYPE_CE_PRI:
+            NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_STATE);
+            break;
+        case TRANSFER_TYPE_BAR0:
+            NV_PRINTF(LEVEL_ERROR, "BAR0 memset unimplemented\n");
+            NV_ASSERT(0);
+            break;
+
     }
 
     return NV_OK;
 }
 
-/*!
- * @brief This function is used to map the appropriate memory descriptor,
- *        copy the memory from the given buffer, and then unmap.
- *
- * @param[in] pMemDesc Memory descriptor of buffer to write
- * @param[in] pBuf     Buffer allocated by caller
- * @param[in] offset   Offset of buffer to write
- * @param[in] size     Size in bytes of the buffer
- * @param[in] flags    Flags
- */
-static NV_STATUS
-memmgrMemWriteMapAndCopy
-(
-    MemoryManager     *pMemoryManager,
-    MEMORY_DESCRIPTOR *pMemDesc,
-    void              *pBuf,
-    NvU64              offset,
-    NvU64              size,
-    NvU32              flags
-)
-{
-    NvU8   *pDst = NULL;
-    OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
-
-    pDst = memdescMapInternal(pGpu, pMemDesc, TRANSFER_FLAGS_NONE);
-    NV_CHECK_OR_RETURN(LEVEL_SILENT, pDst != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
-
-    portMemCopy(pDst + offset, size, pBuf, size);
-    memdescUnmapInternal(pGpu, pMemDesc, flags);
-
-    return NV_OK;
-}
-
-/*!
- * @brief This function is used for writing data placed in a caller passed buffer
- *        to a given memory region while only mapping regions as large as the given
- *        block size.
- *
- * @param[in] pMemDesc   Memory descriptor of buffer to write
- * @param[in] pBuf       Buffer allocated by caller
- * @param[in] baseOffset Offset of entire buffer to write
- * @param[in] size       Size in bytes of the buffer
- * @param[in] flags      Flags
- * @param[in] blockSize  Maximum size of a mapping to use
- */
-static NV_STATUS
-memmgrMemWriteInBlocks
-(
-    MemoryManager     *pMemoryManager,
-    MEMORY_DESCRIPTOR *pMemDesc,
-    void              *pBuf,
-    NvU64              baseOffset,
-    NvU64              size,
-    NvU32              flags,
-    NvU32              blockSize
-)
-{
-    NV_STATUS  status    = NV_OK;
-    OBJGPU    *pGpu      = ENG_GET_GPU(pMemoryManager);
-    NvU64      remaining = size;
-    NvU64      offset    = 0;
-
-    while ((remaining > 0) && (status == NV_OK))
-    {
-        MEMORY_DESCRIPTOR *pSubMemDesc = NULL;
-        NvU32              mapSize     = NV_MIN(blockSize, remaining);
-
-        NV_CHECK_OK_OR_RETURN(LEVEL_SILENT, memdescCreateSubMem(&pSubMemDesc, pMemDesc, pGpu, offset + baseOffset, mapSize));
-
-        // Set the offset to 0, as the sub descriptor already starts at the offset
-        status = memmgrMemWriteMapAndCopy(pMemoryManager, pSubMemDesc, (NvU8 *)pBuf + offset,
-                                          0, mapSize, flags);
-
-        memdescFree(pSubMemDesc);
-        memdescDestroy(pSubMemDesc);
-
-        offset += mapSize;
-        remaining -= mapSize;
-    }
-
-    return status;
-}
-
 /*!
  * @brief This function is used for writing data placed in a caller passed buffer
  *        to a given memory region using the specified memory transfer technique
@@ -682,11 +764,8 @@ memmgrMemWriteWithTransferType
     OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
 
     // Sanitize the input
-    NV_ASSERT_OR_RETURN(pDstInfo != NULL, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pDstInfo->pMemDesc != NULL, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OK_OR_RETURN(memmgrCheckSurfaceBounds(pDstInfo, size));
     NV_ASSERT_OR_RETURN(pBuf != NULL, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(size > 0, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pDstInfo->offset + size <= pDstInfo->pMemDesc->Size, NV_ERR_INVALID_ARGUMENT);
 
     if (pMapping != NULL)
     {
@@ -697,18 +776,18 @@ memmgrMemWriteWithTransferType
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
-            if (memmgrMemWriteMapAndCopy(pMemoryManager, pDstInfo->pMemDesc, pBuf, pDstInfo->offset, size, flags) != NV_OK)
             {
-                // If we fail to map a block large enough for the entire transfer, split up the mapping.
-                NV_ASSERT_OK_OR_RETURN(memmgrMemWriteInBlocks(pMemoryManager, pDstInfo->pMemDesc, pBuf,
-                                                              pDstInfo->offset, size, flags, MEMORY_COPY_BLOCK_SIZE));
+                NvU8 *pDst = memmgrMemBeginTransfer(pMemoryManager, pDstInfo, size, flags);
+                NV_CHECK_OR_RETURN(LEVEL_ERROR, pDst != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
+                portMemCopy(pDst, size, pBuf, size);
+                memmgrMemEndTransfer(pMemoryManager, pDstInfo, size, flags);
             }
             break;
         case TRANSFER_TYPE_GSP_DMA:
             if (IS_GSP_CLIENT(pGpu))
             {
                 NV_PRINTF(LEVEL_INFO, "Calling GSP DMA task\n");
-                NV_ASSERT_OK_OR_RETURN(
+                NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
                     _memmgrMemReadOrWriteWithGsp(pGpu, pDstInfo, pBuf, size,
                                                  NV_FALSE /* bRead */));
             }
@@ -718,7 +797,11 @@ memmgrMemWriteWithTransferType
             }
             break;
         case TRANSFER_TYPE_CE:
-            NV_PRINTF(LEVEL_INFO, "Add call to CE\n");
+        case TRANSFER_TYPE_CE_PRI:
+        case TRANSFER_TYPE_BAR0:
+            NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                _memmgrMemReadOrWriteUsingStagingBuffer(pMemoryManager, pDstInfo, pBuf, size,
+                                                        transferType, NV_FALSE /* bRead */));
             break;
     }
 
@@ -748,16 +831,11 @@ memmgrMemReadWithTransferType
 )
 {
     OBJGPU *pGpu = ENG_GET_GPU(pMemoryManager);
-    NvU8   *pSrc;
     NvU8   *pMapping = memdescGetKernelMapping(pSrcInfo->pMemDesc);
 
-
     // Sanitize the input
-    NV_ASSERT_OR_RETURN(pSrcInfo != NULL, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pSrcInfo->pMemDesc != NULL, NV_ERR_INVALID_ARGUMENT);
+    NV_ASSERT_OK_OR_RETURN(memmgrCheckSurfaceBounds(pSrcInfo, size));
     NV_ASSERT_OR_RETURN(pBuf != NULL, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(size > 0, NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pSrcInfo->offset + size <= pSrcInfo->pMemDesc->Size, NV_ERR_INVALID_ARGUMENT);
 
     if (pMapping != NULL)
     {
@@ -768,17 +846,17 @@ memmgrMemReadWithTransferType
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
-            pSrc = memdescMapInternal(pGpu, pSrcInfo->pMemDesc, TRANSFER_FLAGS_NONE);
-            NV_ASSERT_OR_RETURN(pSrc != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
-
-            portMemCopy(pBuf, size, pSrc + pSrcInfo->offset, size);
-
-            memdescUnmapInternal(pGpu, pSrcInfo->pMemDesc, 0);
+            {
+                NvU8 *pSrc = memmgrMemBeginTransfer(pMemoryManager, pSrcInfo, size, flags);
+                NV_CHECK_OR_RETURN(LEVEL_INFO, pSrc != NULL, NV_ERR_INSUFFICIENT_RESOURCES);
+                portMemCopy(pBuf, size, pSrc, size);
+                memmgrMemEndTransfer(pMemoryManager, pSrcInfo, size, flags);
+            }
             break;
         case TRANSFER_TYPE_GSP_DMA:
             if (IS_GSP_CLIENT(pGpu))
             {
-                NV_ASSERT_OK_OR_RETURN(
+                NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
                     _memmgrMemReadOrWriteWithGsp(pGpu, pSrcInfo, pBuf, size,
                                                  NV_TRUE /* bRead */));
             }
@@ -788,7 +866,11 @@ memmgrMemReadWithTransferType
             }
             break;
         case TRANSFER_TYPE_CE:
-            NV_PRINTF(LEVEL_INFO, "Add call to CE\n");
+        case TRANSFER_TYPE_CE_PRI:
+        case TRANSFER_TYPE_BAR0:
+            NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                _memmgrMemReadOrWriteUsingStagingBuffer(pMemoryManager, pSrcInfo, pBuf, size,
+                                                        transferType, NV_TRUE /* bRead */));
             break;
     }
 
@@ -918,6 +1000,66 @@ memmgrMemUtilsGetMemDescFromHandle_IMPL
     return pMemory->pMemDesc;
 }
 
+/*!
+ * @brief This function is used for write a value placed in a caller passed buffer
+ *        to a given memory region while only mapping regions as large as the given
+ *        block size.
+ *
+ * @param[in] pMemDesc   Memory descriptor of buffer to write
+ * @param[in] value      Value to be written
+ * @param[in] baseOffset Offset of entire buffer to write
+ * @param[in] size       Size in bytes of the buffer
+ * @param[in] flags      Flags
+ * @param[in] blockSize  Maximum size of a mapping to use
+ */
+NV_STATUS
+memmgrMemsetInBlocks_IMPL
+(
+    MemoryManager     *pMemoryManager,
+    MEMORY_DESCRIPTOR *pMemDesc,
+    NvU32              value,
+    NvU64              baseOffset,
+    NvU64              size,
+    NvU32              flags,
+    NvU32              blockSize
+)
+{
+    NV_STATUS  status    = NV_OK;
+    OBJGPU    *pGpu      = ENG_GET_GPU(pMemoryManager);
+    NvU64      remaining = size;
+    NvU64      offset    = 0;
+
+    if (blockSize == 0)
+    {
+        blockSize = MEMORY_COPY_BLOCK_SIZE;
+    }
+
+    while ((remaining > 0) && (status == NV_OK))
+    {
+        MEMORY_DESCRIPTOR *pSubMemDesc = NULL;
+        NvU32              mapSize     = NV_MIN(blockSize, remaining);
+        TRANSFER_SURFACE   surf = {0};
+
+        NV_CHECK_OK_OR_RETURN(LEVEL_SILENT, memdescCreateSubMem(&pSubMemDesc, pMemDesc, pGpu, offset + baseOffset, mapSize));
+
+        surf.pMemDesc = pSubMemDesc;
+        surf.offset = 0;
+
+        NV_ASSERT_OK_OR_RETURN(
+            memmgrMemSet(pMemoryManager, &surf, value,
+                         pSubMemDesc->Size,
+                         flags));
+
+        memdescFree(pSubMemDesc);
+        memdescDestroy(pSubMemDesc);
+
+        offset += mapSize;
+        remaining -= mapSize;
+    }
+
+    return status;
+}
+
 /*!
  * @brief This function is used for copying data b/w two memory regions
  *        Both memory regions can be in the same aperture of different apertures
@@ -938,7 +1080,7 @@ memmgrMemCopy_IMPL
 )
 {
     TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager,
-                                                          pDstInfo, pSrcInfo);
+                                                          pDstInfo, pSrcInfo, flags);
 
     return memmgrMemCopyWithTransferType(pMemoryManager, pDstInfo, pSrcInfo,
                                          size, transferType, flags);
@@ -963,7 +1105,7 @@ memmgrMemSet_IMPL
 )
 {
     TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager,
-                                                          pDstInfo, NULL);
+                                                          pDstInfo, NULL, flags);
 
     return memmgrMemSetWithTransferType(pMemoryManager, pDstInfo, value,
                                         size, transferType, flags);
@@ -987,13 +1129,75 @@ memmgrMemDescMemSet_IMPL
 {
     TRANSFER_SURFACE transferSurface = {.offset = 0, .pMemDesc = pMemDesc};
     TRANSFER_TYPE    transferType = memmgrGetMemTransferType(pMemoryManager,
-                                                             &transferSurface, NULL);
+                                                             &transferSurface, NULL, flags);
 
     return memmgrMemSetWithTransferType(pMemoryManager, &transferSurface, value,
                                         (NvU32)memdescGetSize(pMemDesc),
                                         transferType, flags);
 }
 
+static NV_STATUS
+memmgrMemReadOrWriteInBlocks
+(
+    MemoryManager    *pMemoryManager,
+    TRANSFER_SURFACE *pDstInfo,
+    void             *pBuf,
+    NvU64             size,
+    TRANSFER_TYPE     transferType,
+    NvU32             flags,
+    NvBool            bRead
+)
+{
+    NvBool             bCreateSubMemDesc = (transferType == TRANSFER_TYPE_PROCESSOR) &&
+                                           !(flags & TRANSFER_FLAGS_USE_BAR1);
+    NvU64              remainingSize     = size;
+    NvU64              baseOffset        = pDstInfo->offset;
+    MEMORY_DESCRIPTOR *pMemDesc          = pDstInfo->pMemDesc;
+    NvU64              offset            = 0;
+    NV_STATUS          status            = NV_OK;
+
+    while ((remainingSize > 0) && (status == NV_OK))
+    {
+        MEMORY_DESCRIPTOR *pSubMemDesc = NULL;
+        NvU64              copySize    = NV_MIN(MEMORY_COPY_BLOCK_SIZE, remainingSize);
+
+        if (bCreateSubMemDesc)
+        {
+            NV_ASSERT_OK_OR_RETURN(
+                memdescCreateSubMem(&pSubMemDesc, pMemDesc, pMemDesc->pGpu, offset + baseOffset, copySize));
+            pDstInfo->pMemDesc = pSubMemDesc;
+            pDstInfo->offset = 0;
+        }
+        else
+        {
+            pDstInfo->offset = offset + baseOffset;
+        }
+
+        if (bRead)
+        {
+            status = memmgrMemReadWithTransferType(pMemoryManager, pDstInfo, (NvU8 *)pBuf + offset,
+                                                   copySize, transferType, flags);
+        }
+        else
+        {
+            status = memmgrMemWriteWithTransferType(pMemoryManager, pDstInfo, (NvU8 *)pBuf + offset,
+                                                    copySize, transferType, flags);
+        }
+        NV_ASSERT_OK(status);
+
+        if (bCreateSubMemDesc)
+        {
+            memdescFree(pSubMemDesc);
+            memdescDestroy(pSubMemDesc);
+        }
+
+        offset += copySize;
+        remainingSize -= copySize;
+    }
+
+    return status;
+}
+
 /*!
  * @brief This function is used for writing data placed in a user buffer
  *        to a given memory region
@@ -1014,10 +1218,15 @@ memmgrMemWrite_IMPL
 )
 {
     TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager,
-                                                          pDstInfo, NULL);
+                                                          pDstInfo, NULL, flags);
 
-    return memmgrMemWriteWithTransferType(pMemoryManager, pDstInfo, pBuf,
-                                          size, transferType, flags);
+    NV_STATUS status = memmgrMemWriteWithTransferType(pMemoryManager, pDstInfo, pBuf,
+                                                      size, transferType, flags);
+    if (status == NV_OK)
+        return NV_OK;
+
+    return memmgrMemReadOrWriteInBlocks(pMemoryManager, pDstInfo, pBuf,
+                                        size, transferType, flags, NV_FALSE);
 }
 
 /*!
@@ -1040,10 +1249,15 @@ memmgrMemRead_IMPL
 )
 {
     TRANSFER_TYPE transferType = memmgrGetMemTransferType(pMemoryManager,
-                                                          NULL, pSrcInfo);
+                                                          NULL, pSrcInfo, flags);
 
-    return memmgrMemReadWithTransferType(pMemoryManager, pSrcInfo, pBuf,
-                                         size, transferType, flags);
+    NV_STATUS status = memmgrMemReadWithTransferType(pMemoryManager, pSrcInfo, pBuf,
+                                                     size, transferType, flags);
+    if (status == NV_OK)
+        return NV_OK;
+
+    return memmgrMemReadOrWriteInBlocks(pMemoryManager, pSrcInfo, pBuf,
+                                        size, transferType, flags, NV_TRUE);
 }
 
 /*!
@@ -1063,19 +1277,16 @@ memmgrMemBeginTransfer_IMPL
 )
 {
     TRANSFER_TYPE      transferType = memmgrGetMemTransferType(pMemoryManager,
-                                                               pTransferInfo, NULL);
+                                                               pTransferInfo, NULL, flags);
     MEMORY_DESCRIPTOR *pMemDesc     = pTransferInfo->pMemDesc;
     NvU64              offset       = pTransferInfo->offset;
     OBJGPU            *pGpu         = ENG_GET_GPU(pMemoryManager);
     NvU8              *pPtr         = NULL;
-    NvU64              memSz        = 0;
+    NvU64              memSz        = shadowBufSize;
 
-    NV_ASSERT_OR_RETURN(pMemDesc != NULL, NULL);
-    NV_ASSERT_OR_RETURN((memSz = memdescGetSize(pMemDesc)) >= shadowBufSize, NULL);
+    NV_ASSERT_OR_RETURN(memmgrCheckSurfaceBounds(pTransferInfo, memSz) == NV_OK, NULL);
     NV_ASSERT_OR_RETURN(memdescGetKernelMapping(pMemDesc) == NULL, NULL);
 
-    memSz = shadowBufSize == 0 ? memSz : shadowBufSize;
-
     switch (transferType)
     {
         case TRANSFER_TYPE_PROCESSOR:
@@ -1093,17 +1304,19 @@ memmgrMemBeginTransfer_IMPL
                     protect = NV_PROTECT_WRITEABLE;
                 }
 
-                NV_ASSERT_OR_RETURN(memdescMap(pMemDesc, offset, memSz, NV_TRUE, protect,
+                NV_CHECK_OR_RETURN(LEVEL_ERROR, memdescMap(pMemDesc, offset, memSz, NV_TRUE, protect,
                     (NvP64*) &pPtr, &pPriv) == NV_OK, NULL);
                 memdescSetKernelMappingPriv(pMemDesc, pPriv);
                 break;
             }
-            NV_ASSERT_OR_RETURN((pPtr = memdescMapInternal(pGpu, pMemDesc, flags)) != NULL, NULL);
+            NV_CHECK_OR_RETURN(LEVEL_INFO, (pPtr = memdescMapInternal(pGpu, pMemDesc, flags)) != NULL, NULL);
             pPtr = &pPtr[offset];
 
             break;
         case TRANSFER_TYPE_GSP_DMA:
         case TRANSFER_TYPE_CE:
+        case TRANSFER_TYPE_CE_PRI:
+        case TRANSFER_TYPE_BAR0:
             if (flags & TRANSFER_FLAGS_SHADOW_ALLOC)
             {
                 NV_ASSERT_OR_RETURN((pPtr = portMemAllocNonPaged(memSz)), NULL);
@@ -1137,19 +1350,16 @@ memmgrMemEndTransfer_IMPL
 )
 {
     TRANSFER_TYPE      transferType = memmgrGetMemTransferType(pMemoryManager,
-                                                               pTransferInfo, NULL);
+                                                               pTransferInfo, NULL, flags);
     MEMORY_DESCRIPTOR *pMemDesc     = pTransferInfo->pMemDesc;
-    NvU64              offset       = pTransferInfo->offset;
     OBJGPU            *pGpu         = ENG_GET_GPU(pMemoryManager);
-    NvU64              memSz        = 0;
     NvU8              *pMapping     = NULL;
+    NvU64              memSz        = shadowBufSize;
+
+    NV_ASSERT_OR_RETURN_VOID(memmgrCheckSurfaceBounds(pTransferInfo, memSz) == NV_OK);
 
-    NV_ASSERT_OR_RETURN_VOID(pMemDesc != NULL);
     pMapping = memdescGetKernelMapping(pMemDesc);
 
-    NV_ASSERT_OR_RETURN_VOID((memSz = memdescGetSize(pMemDesc)) >= (shadowBufSize + offset) );
-    memSz = shadowBufSize == 0 ? memSz : shadowBufSize;
-
     memdescSetKernelMapping(pMemDesc, NULL);
 
     switch (transferType)
@@ -1169,6 +1379,8 @@ memmgrMemEndTransfer_IMPL
             return;
         case TRANSFER_TYPE_GSP_DMA:
         case TRANSFER_TYPE_CE:
+        case TRANSFER_TYPE_CE_PRI:
+        case TRANSFER_TYPE_BAR0:
             if (pMapping != NULL)
             {
                 NV_ASSERT_OK(memmgrMemWrite(pMemoryManager, pTransferInfo, pMapping, memSz, flags));
@@ -1444,6 +1656,11 @@ memUtilsAllocMemDesc
     {
         NvU64 memDescFlags = MEMDESC_FLAGS_SKIP_RESOURCE_COMPUTE;
 
+        if (FLD_TEST_DRF(OS32, _ATTR2, _USE_EGM, _TRUE, pFbAllocInfo->retAttr2))
+        {
+            memDescFlags |= MEMDESC_FLAGS_ALLOC_FROM_EGM;
+        }
+
         //
         // Allocate a contig vidmem descriptor now; if needed we'll
         // allocate a new noncontig memdesc later
