@@ -62,6 +62,24 @@ subdeviceCtrlCmdBusGetNvlinkCaps_IMPL
     return nvlinkCtrlCmdBusGetNvlinkCaps(pGpu, pParams);
 }
 
+NV_STATUS
+subdeviceCtrlCmdBusGetNvlinkCaps_VF
+(
+    Subdevice *pSubdevice,
+    NV2080_CTRL_CMD_NVLINK_GET_NVLINK_CAPS_PARAMS *pParams
+)
+{
+    OBJGPU *pGpu = GPU_RES_GET_GPU(pSubdevice);
+    VGPU_STATIC_INFO *pVSI = GPU_GET_STATIC_INFO(pGpu);
+
+    NV_ASSERT_OR_RETURN(pVSI != NULL, NV_ERR_INVALID_STATE);
+
+    portMemCopy(pParams, sizeof(*pParams),
+                &pVSI->nvlinkCaps, sizeof(pVSI->nvlinkCaps));
+
+    return NV_OK;
+}
+
 static void _calculateNvlinkCaps
 (
     OBJGPU *pGpu,
@@ -69,6 +87,7 @@ static void _calculateNvlinkCaps
     NvU32   bridgedLinks,
     NvU32   ipVerNvlink,
     NvBool  bMIGNvLinkP2PSupported,
+    NvBool  bNvlinkEnabled,
     NV2080_CTRL_CMD_NVLINK_GET_NVLINK_CAPS_PARAMS *pParams
 )
 {
@@ -179,6 +198,11 @@ static void _calculateNvlinkCaps
         }
     }
 
+    if (bNvlinkEnabled)
+    {
+        RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _VALID);
+    }
+
     portMemCopy(&pParams->capsTbl, NV2080_CTRL_NVLINK_CAPS_TBL_SIZE, tempCaps, NV2080_CTRL_NVLINK_CAPS_TBL_SIZE);
 }
 
@@ -197,23 +221,6 @@ nvlinkCtrlCmdBusGetNvlinkCaps
     KernelMIGManager *pKernelMIGManager      = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
     NvBool            bMIGNvLinkP2PSupported = ((pKernelMIGManager != NULL) &&
                                                 kmigmgrIsMIGNvlinkP2PSupported(pGpu, pKernelMIGManager));
-    //
-    // vGPU:
-    //
-    // Since vGPU does all real hardware management in the
-    // host, if we are in guest OS (where IS_VIRTUAL(pGpu) is true),
-    // do an RPC to the host to get blacklist information from host RM
-    //
-    if (IS_VIRTUAL(pGpu))
-    {
-        CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
-        RmCtrlParams *pRmCtrlParams = pCallContext->pControlParams;
-        NV_STATUS     status = NV_OK;
-
-        NV_RM_RPC_CONTROL(pGpu, pRmCtrlParams->hClient, pRmCtrlParams->hObject, pRmCtrlParams->cmd,
-                          pRmCtrlParams->pParams, pRmCtrlParams->paramsSize, status);
-        return status;
-    }
 
     // Initialize link masks to 0
     pParams->enabledLinkMask    = 0;
@@ -241,7 +248,10 @@ nvlinkCtrlCmdBusGetNvlinkCaps
             //
             knvlinkFilterBridgeLinks_HAL(pGpu, pKernelNvlink);
         }
-        _calculateNvlinkCaps(pGpu, pKernelNvlink->bridgeSensableLinks, pKernelNvlink->bridgedLinks, pKernelNvlink->ipVerNvlink, bMIGNvLinkP2PSupported, pParams);
+        _calculateNvlinkCaps(pGpu, pKernelNvlink->bridgeSensableLinks, pKernelNvlink->bridgedLinks,
+                             pKernelNvlink->ipVerNvlink, bMIGNvLinkP2PSupported,
+                             pKernelNvlink->getProperty(pNvlink, PDB_PROP_KNVLINK_ENABLED),
+                             pParams);
 
         pParams->discoveredLinkMask = knvlinkGetDiscoveredLinkMask(pGpu, pKernelNvlink);
         pParams->enabledLinkMask    = knvlinkGetEnabledLinkMask(pGpu, pKernelNvlink);
@@ -333,7 +343,6 @@ static _getNvlinkStatus
             case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_2_0:
                 RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _POWER_STATE_L0);
                 break;
-
             case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_4_0:
             case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_3_1:
             case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_3_0:
@@ -430,6 +439,10 @@ static _getNvlinkStatus
         pParams->linkInfo[i].nvlinkRefClkType        = pLinkAndClockValues->nvlinkRefClkType;
         pParams->linkInfo[i].nvlinkRefClkSpeedMhz    = pLinkAndClockValues->nvlinkReqLinkClockMhz;
 
+        pParams->linkInfo[i].nvlinkMinL1Threshold    = pLinkAndClockValues->nvlinkMinL1Threshold;
+        pParams->linkInfo[i].nvlinkMaxL1Threshold    = pLinkAndClockValues->nvlinkMaxL1Threshold;
+        pParams->linkInfo[i].nvlinkL1ThresholdUnits  = pLinkAndClockValues->nvlinkL1ThresholdUnits;
+
         if (nvlinkLinks[i].bConnected)
         {
             pParams->linkInfo[i].connected              = NV2080_CTRL_NVLINK_STATUS_CONNECTED_TRUE;
@@ -624,6 +637,8 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
                 FOR_EACH_INDEX_IN_MASK(32, i, pParams->enabledLinkMask)
                 {
                     NV2080_CTRL_NVLINK_DEVICE_INFO *pDeviceInfo = &pParams->linkInfo[i].remoteDeviceInfo;
+                    if (pDeviceInfo->deviceType == NV2080_CTRL_NVLINK_DEVICE_INFO_DEVICE_TYPE_SWITCH)
+                    	continue;
                     OBJGPU *pLoopGpu = gpumgrGetGpuFromUuid(pDeviceInfo->deviceUUID,
                                                             DRF_DEF(2080_GPU_CMD, _GPU_GET_GID_FLAGS, _TYPE, _SHA1) |
                                                             DRF_DEF(2080_GPU_CMD, _GPU_GET_GID_FLAGS, _FORMAT, _BINARY));
@@ -659,7 +674,7 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
 
     if (!bMIGNvLinkP2PSupported)
     {
-        NV_PRINTF(LEVEL_ERROR, "MIG NVLink P2P is not supported.\n");
+        NV_PRINTF(LEVEL_INFO, "MIG NVLink P2P is not supported.\n");
         status = NV_OK;
         return status;
     }
@@ -722,7 +737,7 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
         pParams->enabledLinkMask = (bIsNvlinkReady) ? pKernelNvlink->enabledLinks : 0x0;
 
         pTmpData->nvlinkLinkAndClockInfoParams.linkMask = pParams->enabledLinkMask;
-        pTmpData->nvlinkLinkAndClockInfoParams.bSublinkStateInst = NV_TRUE;
+        pTmpData->nvlinkLinkAndClockInfoParams.bSublinkStateInst = pParams->bSublinkStateInst;
 
         status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
                                      NV2080_CTRL_CMD_NVLINK_GET_LINK_AND_CLOCK_INFO,
