@@ -301,28 +301,20 @@ krcReportXid_IMPL
         char pid_string[12] = "'<unknown>'";
 
         //
-        // When the channel context is available, use the allocating process'
-        // PID/name, which were populated when the channel was allocated.
-        // For errors without a channel context, use the current process
-        // information, if there's a Resource Server CALL_CONTEXT present
-        // in TLS, indicating we're servicing an RMAPI request.
+        // Get PID of channel creator if available, or get the current PID for
+        // exception types that never have an associated channel
         //
-        if (pKernelChannel != NULL)
+        // Check for API lock since this can be called from parallel init
+        // path without API lock, and RES_GET_CLIENT requires API lock
+        //
+        if (rmapiLockIsOwner() && (pKernelChannel != NULL))
         {
-            //
-            // This function executes under (at least) the GPU lock, so the
-            // pKernelChannel resource can be accessed, regardless of whether
-            // the API lock is held: the pKernelChannel can't go away while the
-            // GPU lock is held, so the owning client can't go away either.
-            // Duping requires holding all GPU locks, so the client's process/
-            // name can't change in this context either.
-            //
             RsClient *pClient = RES_GET_CLIENT(pKernelChannel);
             RmClient *pRmClient = dynamicCast(pClient, RmClient);
             procname = pRmClient->name;
             nvDbgSnprintf(pid_string, sizeof(pid_string), "%u", pKernelChannel->ProcessID);
         }
-        else if (resservGetTlsCallContext() != NULL)
+        else if (exceptType == GSP_RPC_TIMEOUT)
         {
             NvU32 current_pid = osGetCurrentProcess();
 
