@@ -1,5 +1,5 @@
-/*
- * SPDX-FileCopyrightText: Copyright (c) 2016-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ /*
+ * SPDX-FileCopyrightText: Copyright (c) 2016-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -39,6 +39,7 @@
 
 /* ------------------------------------ Local Defines ------------------------------ */
 #define PMA_CHUNK_SIZE_512M (512 * 1024 * 1024)
+#define PMA_CHUNK_SIZE_4M   (4 * 1024 * 1024)
 #define PMA_CHUNK_SIZE_2M   (2 * 1024 * 1024)
 #define PMA_CHUNK_SIZE_512K (512 * 1024)
 #define PMA_CHUNK_SIZE_256K (256 * 1024)
@@ -81,31 +82,33 @@
  */
 typedef enum
 {
-    RM_POOL_IDX_512M = 0,
-    RM_POOL_IDX_2M   = 1,
-    RM_POOL_IDX_256K = 2,
-    RM_POOL_IDX_128K = 3,
-    RM_POOL_IDX_64K  = 4,
-    RM_POOL_IDX_8K   = 5,
-    RM_POOL_IDX_4K   = 6,
-    RM_POOL_IDX_256B = 7,
+    RM_POOL_IDX_512M,
+    RM_POOL_IDX_2M,
+    RM_POOL_IDX_256K,
+    RM_POOL_IDX_128K,
+    RM_POOL_IDX_64K,
+    RM_POOL_IDX_8K,
+    RM_POOL_IDX_4K,
+    RM_POOL_IDX_256B,
     NUM_POOLS          // This should always be the last entry!
 }POOL_IDX;
 
 /*!
  * This array contains the alloction sizes (in bytes) of each pool.
  */
-static const NvU32 poolAllocSizes[] = {0x20000000, 0x200000, 0x40000, 0x20000, 0x10000, 0x2000, 0x1000, 0x100};
+static const NvU64 poolAllocSizes[] = {
+    0x20000000, 0x200000, 0x40000, 0x20000, 0x10000, 0x2000, 0x1000, 0x100
+};
 
 #define POOL_CONFIG_POOL_IDX       0
 #define POOL_CONFIG_CHUNKSIZE_IDX  1
 
-static const int poolConfig[POOL_CONFIG_MAX_SUPPORTED][POOL_CONFIG_CHUNKSIZE_IDX + 1] = {
+static const NvU64 poolConfig[POOL_CONFIG_MAX_SUPPORTED][POOL_CONFIG_CHUNKSIZE_IDX + 1] = {
      // page size        // chunk size
      { RM_POOL_IDX_256K, PMA_CHUNK_SIZE_512K},  // pool with pageSize = 256K for GMMU_FMT_VERSION_1
      { RM_POOL_IDX_4K,   PMA_CHUNK_SIZE_64K },  // pool with pageSize = 4K for GMMU_FMT_VERSION_2
      { RM_POOL_IDX_512M, PMA_CHUNK_SIZE_512M }, // pool with pageSize = 512MB for RM allocated buffers (unused as of ampere)
-     { RM_POOL_IDX_2M,   PMA_CHUNK_SIZE_2M },   // pool with pageSize = 2MB for RM allocated buffers
+     { RM_POOL_IDX_2M,   PMA_CHUNK_SIZE_4M },   // pool with pageSize = 2MB for RM allocated buffers
      { RM_POOL_IDX_64K,  PMA_CHUNK_SIZE_256K }, // pool with pageSize = 64K for RM allocated buffers
      { RM_POOL_IDX_4K,   PMA_CHUNK_SIZE_64K }   // pool with pageSize = 4K for RM allocated buffers
 };
@@ -177,7 +180,7 @@ struct RM_POOL_ALLOC_MEM_RESERVE_INFO
      */
     NvBool bTrimOnFree;
 
-    /*! 
+    /*!
      * Allocate pool in protected memory
      */
     NvBool bProtected;
@@ -190,7 +193,7 @@ struct RM_POOL_ALLOC_MEM_RESERVE_INFO
  *        pool.
  *
  * @param[in] pCtx     Context for upstream allocator.
- * @param[in] pageSize Only for debugging.
+ * @param[in] pageSize Page size to use when allocating from PMA
  * @param[in] pPage    Output page handle from upstream.
  *
  * @return NV_STATUS
@@ -200,20 +203,20 @@ allocUpstreamTopPool
 (
     void             *pCtx,
     NvU64             pageSize,
+    NvU64             numPages, 
     POOLALLOC_HANDLE *pPage
 )
 {
     PMA_ALLOCATION_OPTIONS      allocOptions = {0};
     RM_POOL_ALLOC_MEM_RESERVE_INFO *pMemReserveInfo;
-    NV_STATUS                   status;
+    NvU64 i, pageBegin;
+    NV_STATUS status;
 
     NV_ASSERT_OR_RETURN(NULL != pCtx, NV_ERR_INVALID_ARGUMENT);
     NV_ASSERT_OR_RETURN(NULL != pPage, NV_ERR_INVALID_ARGUMENT);
 
-    // TODO: Replace the direct call to PMA with function pointer.
     pMemReserveInfo = (RM_POOL_ALLOC_MEM_RESERVE_INFO *)pCtx;
-    allocOptions.flags = PMA_ALLOCATE_PINNED | PMA_ALLOCATE_PERSISTENT |
-                         PMA_ALLOCATE_CONTIGUOUS;
+    allocOptions.flags = PMA_ALLOCATE_PINNED | PMA_ALLOCATE_PERSISTENT;
 
     if (pMemReserveInfo->bSkipScrub)
     {
@@ -225,15 +228,54 @@ allocUpstreamTopPool
         allocOptions.flags |= PMA_ALLOCATE_PROTECTED_REGION;
     }
 
-    status = pmaAllocatePages(pMemReserveInfo->pPma,
-                              (NvU32)(pMemReserveInfo->pmaChunkSize/PMA_CHUNK_SIZE_64K),
-                              PMA_CHUNK_SIZE_64K,
-                              &allocOptions,
-                              &pPage->address);
-    NV_ASSERT_OR_RETURN((NV_OK == status), status);
+    //
+    // Some tests fail page table and directory allocation when close to all FB is allocated if we allocate contiguously.
+    // For now, we're special-casing this supported pageSize and allocating the 64K pages discontigously.
+    // TODO: Unify the codepaths so that all pages allocated discontiguously (not currently supported by PMA)
+    //
+    if (pageSize == PMA_CHUNK_SIZE_64K)
+    {
+        NvU64 *pPageStore = portMemAllocNonPaged(sizeof(NvU64) * numPages);
+        NV_STATUS status = NV_OK;
+        NV_ASSERT_OK_OR_GOTO(status,
+            pmaAllocatePages(pMemReserveInfo->pPma,
+                numPages,
+                pageSize,
+                &allocOptions,
+                pPageStore),
+            free_mem);
 
-    pPage->pMetadata = NULL;
+        for (i = 0; i < numPages; i++)
+        {
+            pPage[i].address = pPageStore[i];
+            pPage[i].pMetadata = NULL;
+        }
+free_mem:
+        portMemFree(pPageStore);
+        return status;
+    }
 
+    allocOptions.flags |= PMA_ALLOCATE_CONTIGUOUS;
+
+    for (i = 0; i < numPages; i++)
+    {
+        NV_ASSERT_OK_OR_GOTO(status, pmaAllocatePages(pMemReserveInfo->pPma,
+            pageSize / PMA_CHUNK_SIZE_64K,
+            PMA_CHUNK_SIZE_64K,
+            &allocOptions,
+            &pageBegin), err);
+        pPage[i].address = pageBegin;
+        pPage[i].pMetadata = NULL;
+    }
+
+    return NV_OK;
+err:
+    for (;i > 0; i--)
+    {
+        NvU32 flags = pMemReserveInfo->bSkipScrub ? PMA_FREE_SKIP_SCRUB : 0;
+        pmaFreePages(pMemReserveInfo->pPma, &pPage[i - 1].address, 1,
+            pageSize, flags);
+    }
     return status;
 }
 
@@ -252,17 +294,28 @@ allocUpstreamLowerPools
 (
     void             *pCtx,
     NvU64             pageSize,
+    NvU64             numPages,
     POOLALLOC_HANDLE *pPage
 )
 {
     NV_STATUS status;
+    NvU64 i;
 
     NV_ASSERT_OR_RETURN(NULL != pCtx, NV_ERR_INVALID_ARGUMENT);
     NV_ASSERT_OR_RETURN(NULL != pPage, NV_ERR_INVALID_ARGUMENT);
 
-    status = poolAllocate((POOLALLOC *)pCtx, pPage);
-    NV_ASSERT_OR_RETURN(status == NV_OK, status);
-
+    for(i = 0; i < numPages; i++)
+    {
+        NV_ASSERT_OK_OR_GOTO(status,
+            poolAllocate((POOLALLOC *)pCtx, &pPage[i]),
+            cleanup);
+    }
+    return NV_OK;
+cleanup:
+    for(;i > 0; i--)
+    {
+        poolFree((POOLALLOC *)pCtx, &pPage[i-1]);
+    }
     return status;
 }
 
@@ -428,14 +481,11 @@ rmMemPoolSetup
     // The topmost pool is fed pages directly by PMA.
     //
     // Calling into PMA with GPU lock acquired may cause deadlocks in case RM
-    // is operating along side UVM. Currently, we don't support UVM on Windows.
-    // So, allow the topmost pool to call into PMA on Windows. This is not
+    // is operating along side UVM. Currently, we don't support UVM on Windows/MODS.
+    // So, allow the topmost pool to call into PMA on Windows/MODS. This is not
     // permissible on platforms that support UVM like Linux.
-    // TODO: Remove this special handling for Windows once we have taken care
-    // of reserving memory for page tables required for mapping GR context buffers
-    // in the channel vaspace. See bug 200590870 and 200614517.
     //
-    if (RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM)
+    if (RMCFG_FEATURE_PLATFORM_WINDOWS || RMCFG_FEATURE_PLATFORM_MODS)
     {
         flags = FLD_SET_DRF(_RMPOOL, _FLAGS, _AUTO_POPULATE, _ENABLE, flags);
     }
@@ -444,7 +494,7 @@ rmMemPoolSetup
         flags = FLD_SET_DRF(_RMPOOL, _FLAGS, _AUTO_POPULATE, _DISABLE, flags);
     }
     pMemReserveInfo->pPool[pMemReserveInfo->topmostPoolIndex] = poolInitialize(
-                                                 (NvU32)pMemReserveInfo->pmaChunkSize,
+                                                 pMemReserveInfo->pmaChunkSize,
                                                  poolAllocSizes[pMemReserveInfo->topmostPoolIndex],
                                                  allocUpstreamTopPool,
                                                  freeUpstreamTopPool,
@@ -611,7 +661,7 @@ rmMemPoolAllocate
             if (allocSize <= poolAllocSizes[poolIndex])
             {
                 NV_PRINTF(LEVEL_INFO,
-                    "Allocating from pool with alloc size = 0x%x Bytes\n",
+                    "Allocating from pool with alloc size = 0x%llx Bytes\n",
                     poolAllocSizes[poolIndex]);
                 break;
             }
@@ -633,7 +683,7 @@ rmMemPoolAllocate
         NvU32 index;
 
         NV_PRINTF(LEVEL_INFO,
-            "Allocating from pool with alloc size = 0x%x Bytes\n",
+            "Allocating from pool with alloc size = 0x%llx Bytes\n",
             poolAllocSizes[topPool] * numPages);
 
         if (memdescGetContiguity(pMemDesc, AT_GPU))
@@ -692,8 +742,7 @@ rmMemPoolAllocate
         if (status != NV_OK)
         {
             listRemove(pPageHandleList, pPageHandle);
-            NV_ASSERT_OR_GOTO((NV_OK == status), done);
-            pPageHandle = NULL;
+            goto done;
         }
 
         memdescDescribe(pMemDesc, ADDR_FBMEM, pPageHandle->address, pMemDesc->Size);
@@ -981,7 +1030,7 @@ rmMemPoolGetChunkAndPageSize
 (
     RM_POOL_ALLOC_MEM_RESERVE_INFO *pMemReserveInfo,
     NvU64 *pChunkSize,
-    NvU32 *pPageSize
+    NvU64 *pPageSize
 )
 {
     NV_ASSERT_OR_RETURN(pMemReserveInfo != NULL, NV_ERR_INVALID_ARGUMENT);
