@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2000-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2000-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -34,6 +34,7 @@
 #include "gps.h"
 #include "nvop.h"
 #include "nvhybridacpi.h"
+#include "nvmisc.h"
 #include "jt.h"
 #include "pex.h"
 #include "mxm_spec.h"
@@ -358,13 +359,13 @@ _getRemappedDsmSubfunction
     // Use NV_ASSERT_OR_ELSE_STR for embedded %.
     //
     NV_ASSERT_OR_ELSE_STR(
-        !(NV_ARRAY_ELEMENTS32(genDsmSubFuncRemapTable) % (ACPI_DSM_FUNCTION_COUNT + 1)),
-        "!(NV_ARRAY_ELEMENTS32(genDsmSubFuncRemapTable) %% (ACPI_DSM_FUNCTION_COUNT + 1))",
+        !(NV_ARRAY_ELEMENTS(genDsmSubFuncRemapTable) % (ACPI_DSM_FUNCTION_COUNT + 1)),
+        "!(NV_ARRAY_ELEMENTS(genDsmSubFuncRemapTable) %% (ACPI_DSM_FUNCTION_COUNT + 1))",
         return NV_ERR_INVALID_ARGUMENT);
 
     // find the event in the table
     i = 0;
-    while (i <= (NV_ARRAY_ELEMENTS32(genDsmSubFuncRemapTable) - ACPI_DSM_FUNCTION_COUNT - 1))
+    while (i <= (NV_ARRAY_ELEMENTS(genDsmSubFuncRemapTable) - ACPI_DSM_FUNCTION_COUNT - 1))
     {
         if (acpiDsmSubFunction == genDsmSubFuncRemapTable[i])
         {
@@ -475,7 +476,7 @@ remapDsmFunctionAndSubFunction
 
     // Loop through (in precendence order) to find the GUID/subfunction until we find one that supports the call.
     testDSMfuncIndex = 0;
-    while(testDSMfuncIndex < sizeof(dsmOrderOfPrecedenceList)/sizeof(dsmOrderOfPrecedenceList[0]))
+    while(testDSMfuncIndex < NV_ARRAY_ELEMENTS(dsmOrderOfPrecedenceList))
     {
         // remap the generic subfunction number of the one that matches this function.
         if ((_getRemappedDsmSubfunction(dsmOrderOfPrecedenceList[testDSMfuncIndex], *pRemappedDsmSubFunction, &remappedDSMSubFunction)) == NV_OK)
@@ -554,7 +555,7 @@ getDsmGetObjectSubfunction
             if (remapDsmFunctionAndSubFunction(pGpu, &curFuncForGetAllObjects, &dummySubFunc) == NV_OK)
             {
                 testDSMfuncIndex = 0;
-                while(testDSMfuncIndex < sizeof(dsmOrderOfPrecedenceList)/sizeof(dsmOrderOfPrecedenceList[0]))
+                while(testDSMfuncIndex < NV_ARRAY_ELEMENTS(dsmOrderOfPrecedenceList))
                 {
                     if (dsmOrderOfPrecedenceList[testDSMfuncIndex] == curFuncForGetObjByType)
                     {
@@ -823,11 +824,11 @@ _acpiDsmSupportedFuncCacheInit
         // try package type for argument 3.
         pGpu->acpi.dsm[func].bArg3isInteger = NV_FALSE;
         rtnSize = sizeof(supportFuncs);
-        status = pGpu->pOS->osCallACPI_DSM(pGpu,
-                                           func,
-                                           NV_ACPI_ALL_FUNC_SUPPORT,
-                                           (NvU32 *) &supportFuncs,
-                                           &rtnSize);
+        status = osCallACPI_DSM(pGpu,
+                                func,
+                                NV_ACPI_ALL_FUNC_SUPPORT,
+                                (NvU32 *) &supportFuncs,
+                                &rtnSize);
         if (status != NV_OK)
         {
             if (status == NV_ERR_INVALID_ARGUMENT)
@@ -835,11 +836,11 @@ _acpiDsmSupportedFuncCacheInit
                 // maybe an older SBIOS, try integer type for argument 3.
                 pGpu->acpi.dsm[func].bArg3isInteger = NV_TRUE;
                 rtnSize = sizeof(supportFuncs);
-                status = pGpu->pOS->osCallACPI_DSM(pGpu,
-                                                   func,
-                                                   NV_ACPI_ALL_FUNC_SUPPORT,
-                                                   (NvU32 *) &supportFuncs,
-                                                   &rtnSize);
+                status = osCallACPI_DSM(pGpu,
+                                        func,
+                                        NV_ACPI_ALL_FUNC_SUPPORT,
+                                        (NvU32 *) &supportFuncs,
+                                        &rtnSize);
             }
         }
         if (_isDsmError(status, rtnSize, (NvU32 *) &supportFuncs))
@@ -857,7 +858,7 @@ _acpiDsmSupportedFuncCacheInit
     }
 }
 
-static void 
+static void
 _acpiDsmCallbackInit
 (
     OBJGPU *pGpu
@@ -888,7 +889,7 @@ _acpiDsmCallbackInit
 
     // Loop through the list of GUIDs that support callbacks in priority order
     for (testDSMfuncIndex = 0;
-         testDSMfuncIndex < (sizeof(callbackOrderOfPrecedenceList)/sizeof(callbackOrderOfPrecedenceList[0]));
+         testDSMfuncIndex < (NV_ARRAY_ELEMENTS(callbackOrderOfPrecedenceList));
          testDSMfuncIndex++)
     {
         func = callbackOrderOfPrecedenceList[testDSMfuncIndex];
@@ -908,11 +909,11 @@ _acpiDsmCallbackInit
 
         if (testIfDsmSubFunctionEnabled(pGpu, func, NV_ACPI_GENERIC_FUNC_CALLBACKS) == NV_OK)
         {
-            status = pGpu->pOS->osCallACPI_DSM(pGpu,
-                                               func,
-                                               NV_ACPI_GENERIC_FUNC_CALLBACKS,
-                                               &callbacks,
-                                               &rtnSize);
+            status = osCallACPI_DSM(pGpu,
+                                    func,
+                                    NV_ACPI_GENERIC_FUNC_CALLBACKS,
+                                    &callbacks,
+                                    &rtnSize);
 
             if (_isDsmError(status, rtnSize, &callbacks))
             {
@@ -963,7 +964,7 @@ _acpiDsmCallbackInit
  *
  * @returns
  */
-static void 
+static void
 _acpiDsmCapsInit
 (
     OBJGPU *pGpu
@@ -981,11 +982,11 @@ _acpiDsmCapsInit
     if (testIfDsmSubFunctionEnabled(pGpu, func, asmDsmSubFunction) == NV_OK)
     {
         rtnSize = sizeof(platCaps);
-        status = pGpu->pOS->osCallACPI_DSM(pGpu,
-                                           func,
-                                           asmDsmSubFunction,
-                                           &platCaps,
-                                           &rtnSize);
+        status = osCallACPI_DSM(pGpu,
+                                func,
+                                asmDsmSubFunction,
+                                &platCaps,
+                                &rtnSize);
 
         if (_isDsmError(status, rtnSize, &platCaps))
         {
@@ -1108,10 +1109,9 @@ _acpiCacheMethodData
 )
 {
     NV_STATUS status;
-    OBJOS *pOs = GPU_GET_OS(pGpu);
     NvU32 inOut = 0;
     NvU16 rtnSize = sizeof(inOut);
-    NvU32 tableLen = 0, acpiidIndex = 0, mode = 0, muxPartId = 0;
+    NvU32 tableLen = 0, acpiidIndex = 0, mode = 0, muxPartId = 0, state = 0;
 
     // This bit is used for checking if pGpu::acpiMethodData need to be used or not.
     pGpu->acpiMethodData.bValid = NV_TRUE;
@@ -1119,42 +1119,49 @@ _acpiCacheMethodData
     // Fill in the DOD Method Data.
     pGpu->acpiMethodData.dodMethodData.acpiIdListLen = sizeof(pGpu->acpiMethodData.dodMethodData.acpiIdList);
 
-    status = pOs->osCallACPI_DOD(pGpu, pGpu->acpiMethodData.dodMethodData.acpiIdList, &pGpu->acpiMethodData.dodMethodData.acpiIdListLen);
+    status = osCallACPI_DOD(pGpu, pGpu->acpiMethodData.dodMethodData.acpiIdList, &pGpu->acpiMethodData.dodMethodData.acpiIdListLen);
 
     pGpu->acpiMethodData.dodMethodData.status = status;
 
     // Fill in the JT Method Data.
-    status = pOs->osCallACPI_DSM(pGpu, ACPI_DSM_FUNCTION_JT, JT_FUNC_CAPS, &inOut, &rtnSize);
+    status = osCallACPI_DSM(pGpu, ACPI_DSM_FUNCTION_JT, JT_FUNC_CAPS, &inOut, &rtnSize);
 
     pGpu->acpiMethodData.jtMethodData.status  = status;
     pGpu->acpiMethodData.jtMethodData.jtCaps  = inOut;
-    pGpu->acpiMethodData.jtMethodData.jtRevId = (NvU16)DRF_VAL(_JT_FUNC, _CAPS, _REVISION_ID, inOut); 
+    pGpu->acpiMethodData.jtMethodData.jtRevId = (NvU16)DRF_VAL(_JT_FUNC, _CAPS, _REVISION_ID, inOut);
+    gpuSetGC6SBIOSCapabilities(pGpu);
 
     // Fill in the MUX Method Data.
     portMemSet(pGpu->acpiMethodData.muxMethodData.acpiIdMuxModeTable, 0, sizeof(pGpu->acpiMethodData.muxMethodData.acpiIdMuxModeTable));
     portMemSet(pGpu->acpiMethodData.muxMethodData.acpiIdMuxPartTable, 0, sizeof(pGpu->acpiMethodData.muxMethodData.acpiIdMuxPartTable));
+    portMemSet(pGpu->acpiMethodData.muxMethodData.acpiIdMuxStateTable, 0, sizeof(pGpu->acpiMethodData.muxMethodData.acpiIdMuxStateTable));    
     if (pGpu->acpiMethodData.dodMethodData.status == NV_OK)
     {
         tableLen = pGpu->acpiMethodData.dodMethodData.acpiIdListLen / sizeof(NvU32);
         pGpu->acpiMethodData.muxMethodData.tableLen = tableLen;
         for (acpiidIndex = 0; acpiidIndex < tableLen; acpiidIndex++)
         {
-            status = pOs->osCallACPI_MXDM(pGpu, pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex], &mode);
+            status = osCallACPI_MXDM(pGpu, pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex], &mode);
             pGpu->acpiMethodData.muxMethodData.acpiIdMuxModeTable[acpiidIndex].acpiId = pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex];
             pGpu->acpiMethodData.muxMethodData.acpiIdMuxModeTable[acpiidIndex].mode = mode;
             pGpu->acpiMethodData.muxMethodData.acpiIdMuxModeTable[acpiidIndex].status = status;
 
-            status = pOs->osCallACPI_MXID(pGpu, pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex], &muxPartId);
+            status = osCallACPI_MXID(pGpu, pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex], &muxPartId);
             pGpu->acpiMethodData.muxMethodData.acpiIdMuxPartTable[acpiidIndex].acpiId = pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex];
             pGpu->acpiMethodData.muxMethodData.acpiIdMuxPartTable[acpiidIndex].mode = muxPartId;
             pGpu->acpiMethodData.muxMethodData.acpiIdMuxPartTable[acpiidIndex].status = status;
-            mode = muxPartId = 0;
+
+            status = osCallACPI_MXDS(pGpu, pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex], &state);
+            pGpu->acpiMethodData.muxMethodData.acpiIdMuxStateTable[acpiidIndex].acpiId = pGpu->acpiMethodData.dodMethodData.acpiIdList[acpiidIndex];
+            pGpu->acpiMethodData.muxMethodData.acpiIdMuxStateTable[acpiidIndex].mode = state;
+            pGpu->acpiMethodData.muxMethodData.acpiIdMuxStateTable[acpiidIndex].status = status;
+            mode = muxPartId = state = 0;
         }
     }
 
     // Fill in the Optimus caps Method Data.
-    status = pGpu->pOS->osCallACPI_DSM(pGpu, ACPI_DSM_FUNCTION_NVOP, NVOP_FUNC_OPTIMUSCAPS,
-                                        &pGpu->acpiMethodData.capsMethodData.optimusCaps, &rtnSize);
+    status = osCallACPI_DSM(pGpu, ACPI_DSM_FUNCTION_NVOP, NVOP_FUNC_OPTIMUSCAPS,
+                            &pGpu->acpiMethodData.capsMethodData.optimusCaps, &rtnSize);
     pGpu->acpiMethodData.capsMethodData.status = status;
 }
 
